/********************************************************************
   TORNADO-E54x Demo Utility.
   AMD Am29F010B/Am29F040B FLASH test utility. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests on-board AMD Am29F010B/Am29F040B FLASH memory.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
	on-board SW3:
	  - FLASH WRITE ENABLE must be selected
***********************************************************************/

#include	"math.h"

// enable inclusion AMD FLASH utilities into user's code
#define 	__TE54X_INCLUDE_AMD_FLASH_CONTROL__	1
#include	"te54x_d.h"	// include E54x defs


#define	FLASH_1M_TEST_CYCLES		11
#define	FLASH_4M_TEST_CYCLES		15
#define FLASH_CYCLE_LEN			0x100L

unsigned long FLASH_SADDR[15] 	= { 0x00000L, 0x01000L,	
                                    0x02000L, 0x04000L,
                                    0x08000L, 0x08800L,
				    0x10000L, 0x10400L,
                                    0x18000L, 0x18200L,
                                    0x18100L,
				    0x20000L, 0x40000L,
				    0x60000L, 0x70000L		
				  };

unsigned FLASH_SDATA[15]	= { 0x00, 0xe9,
				    0x3f, 0x55,
				    0x25, 0xe3,
				    0x8c, 0xa1,
				    0x66, 0x99,
				    0xbf,
				    0xfa, 0x15,
				    0x79, 0x59
                                   };



void main()
{  int i,j,r;
   unsigned d;
   unsigned long a;
   unsigned manuf_id,dev_id;
   unsigned long len,sectors,sector_len;
   unsigned _flash_test_cycles;

	TE54X_init_for_xdm_flash();

		// get FLASH info
	if((r=TE54X_flash_info(&manuf_id,&dev_id,&len,&sectors,&sector_len))!=TE54X_OK)
             goto error;

	_flash_test_cycles = (dev_id==TE54X_FLASH_AM29F010_DEV_ID) ?
				FLASH_1M_TEST_CYCLES:FLASH_4M_TEST_CYCLES ;

		// erase entire chip with verification
	if((r=TE54X_flash_erase_chip(TE54X_ON))!=TE54X_OK) goto error;

		// now write shifted RAMPS to all predefied FLASH areas
        for(i=0;i<_flash_test_cycles;i++)
          {
            for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
                j<FLASH_CYCLE_LEN;
                a++,j++,d++)
               {
		 if((r=TE54X_flash_write(a,d&TE54X_FLASH_DATA_BITMASK))!=TE54X_OK) goto error;
	       }
	  }

		// now verifies written data
        for(i=0;i<_flash_test_cycles;i++)
          {
            for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
                j<FLASH_CYCLE_LEN;
                a++,j++,d++)
               {
		 if((r=TE54X_flash_read(a))!=(d&TE54X_FLASH_DATA_BITMASK)) goto error;
	       }
	  }

		// finally erase entire chip using sector erase
        for(i=0;i<sectors;i++)
          {
		if((r=TE54X_flash_erase_sector(i*sector_len,TE54X_ON))!=TE54X_OK)
                   goto error;
	  }

halt_ok:
	   /* TRAP here in case programming & verification is OK */
   while(1);	

error:
	   /* TRAP here in case of any error */
   while(1);	


}

