/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   Test for Bootload from FLASH with DRAM test and USART channel-A commuication. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo boots from FLASH/EPROM to DSP on-chip memory and afterthat 
	continuously tests external D-RAM and displays test results via USART
	channel-A, which is configured in ASYNC mode 9600 bauds with RS232C
	external interface. 

	External 9600 baud ASYNC terminal with RS232C interface or 
	PC COM port running in terminal mode at 9600 bauds shall be 
	used with this demo.


   TORNADO platform:
   -----------------
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - FLASH/EPROM BOOT mode
	on-board SW4 switch:
	  - RS232C I/F must be used as external I/F for USART channel-A
	on-board socket S2:
	  - 1.8432 MHz crystal oscillator must be used as USART
	    source/master clock


   Compiling C54x application and conversion to C54x bootloader format:
   ----------------------------------------------------------------------

   C54x applications have to be correctly compiled in order to
   provide compatibility with C5000 Hex Conversion Utility (HEX500.EXE),
   which creates EPROM files in C54x bootloader format.

   For all TORNADO-E54x DSP controllers with C548/C549/C5402/C5409/C5410/C5416
   DSP, all C5000 C/Asm applications shall be compiled with the '-v549'
   or '-v548' command line option:
      cl500 <file> -v549

   This makes HEX500.EXE conversion utility, when reading source COFF .OUT
   file, to generate ROM/PARALLEL boot table using extended addressing
   format, as it described for C5402/C5409/C5410/C5416 bootloaders datasheets
   (SPRA602C, etc). 

   Note, that ROM/PARALLEL boot table format, which is described in SPRU173 
   document (DSP REFERENCE SET: VOLUME 4: APPLICATIONS) is applicable to 
   C541/../C546  DSPs only without extended addressing capabilities.

***********************************************************************/

#include	<math.h>
#include	"te54x_d.h"


#define	ML_OFF			0
#define	ML_ON			1


char welcome_start_e548[]	= "TORNADO-E548";
char welcome_start_e549[]	= "TORNADO-E549";
char welcome_start_e5402[]	= "TORNADO-E5402";
char welcome_start_e5409[]	= "TORNADO-E5409";
char welcome_start_e5410[]	= "TORNADO-E5410";
char welcome_start_e5416[]	= "TORNADO-E5416";
char welcome_start_exxx[]	= "TORNADO-E54??? ";
char welcome_end[]		= " BL-FLASH/SRAM/USART#A test\n\n>>";
char ok_mess[]			= " OK ";
char err_mess[]			= " ERROR ";


usart_a_init()
{
	// configure USART PIO port as outputs
	TE54X_set_usart_rg(TE54X_USART_A_PCR_RG,0);


	// now configure USART channel-A in ASYNC mode with 9600 baud rate
        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,0x0);
	TE54X_set_usart_rg(TE54X_USART_A_MODE_RG,0x0);

        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,0x43); // MCE, ASYNC
	TE54X_set_usart_rg(TE54X_USART_A_CCR1_RG,0x1f); // ODS, BCR, clk_mode=7
	TE54X_set_usart_rg(TE54X_USART_A_CCR2_RG,0x38); // BDF, SSEL (clk_mode=7b), TCLK = out
	TE54X_set_usart_rg(TE54X_USART_A_DAFO_RG,0x00); // STOP=1, PARity=OFF, 8bit 
	TE54X_set_usart_rg(TE54X_USART_A_RFC_RG, 0x00); // RFIFO threshold = 1 
        TE54X_set_usart_rg(TE54X_USART_A_BGR_RG, 0x05);	// 9600 bauds via 1,8432 MHz crystal oscillator

        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_CCR0_RG)| 0x80
			  );				// PU: set channel activity

	TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x41); // receiver/transmitter reset
	TE54X_set_usart_rg(TE54X_USART_A_MODE_RG,0x0c);	// receiver active, set RTS 

	TE54X_set_usart_rg(TE54X_USART_A_IPC_RG, 0x81);	// set unmasked irqs visible, irq active low 
	TE54X_set_usart_rg(TE54X_USART_A_GIS_RG, 0x08);	// enable IRQS from IRQ1/USART-A 
	TE54X_set_usart_rg(TE54X_USART_A_IMR1_RG,~0x21);// enable IRQS from ALLS/XPR
}



#define WAIT_CMDR_READY   {while (TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) &0x4);}

uart_transmit_char(c)
int c;
{
	// transmit using XPR irq with bits ALLS/XPR from ISR1 
  while (!(TE54X_get_usart_rg(TE54X_USART_A_ISR1_RG) &0x21));
  TE54X_set_usart_rg(TE54X_USART_A_TFIFO_RG,c);
  WAIT_CMDR_READY;
  TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x8);
}


int uart_receive_char()
{ int c;

  while (!(TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) &0x20));
  c= TE54X_get_usart_rg(TE54X_USART_A_RFIFO_RG);
  WAIT_CMDR_READY;
  TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x80);	// release RFIFO
  return(c);
}


int uart_receiver_status()
{
  return((TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) & 0x20) ? 1:0);
}


uart_transmit_mess(s)
char *s;
{
  while((*s &0xff)!=0) uart_transmit_char(*(s++));
}




// generates quasi-random next memory data
unsigned next_cell(i)
{
  return( (((~i ^ 0x93a5 -1) <<8)  & 0xff00)
	 |((i+1) & 0xff)  
	);
}


#define ML_OK		0
#define ML_MEM_ERR	-1


// memory test procedure
int mem_test(init_value, sa, ea, dram_pages)
unsigned init_value, *sa, *ea, dram_pages;

{ int r;
  unsigned *ptr,i,page;

	r=ML_MEM_ERR;

	// fill D-RAM

      i=init_value;	
      for(page=0; page<dram_pages; page++)
        {
          TE54X_set_xdm_page(page);

  	  for(ptr=sa; ptr<=ea; ptr++)
	  { *ptr =i;
	    i=next_cell(i);
	  } ;
	}

	// verify written D-RAM data
      i=init_value;	
      for(page=0; page<dram_pages; page++)
        {
          TE54X_set_xdm_page(page);

          for(ptr=sa; ptr<=ea; ptr++)
	  { if ((*ptr)!=i) return (r);
	    i=next_cell(i);
	  }
	}

      // test is OK
      return(ML_OK);
}





void main()
{
	unsigned iv, *start, *end, dram_pages;

	// initialize DSP for running with SRAM as D-MEM
	TE54X_init_for_xdm_dram();

	// init USART channel-A
	usart_a_init();

	// transmit greeting message
	switch(TE54X_get_dev_id())
	{
	  case TE548_DEV_ID:
	        uart_transmit_mess(welcome_start_e548);
		break;

	  case TE549_DEV_ID:
	        uart_transmit_mess(welcome_start_e549);
		break;

	  case TE5402_DEV_ID:
	        uart_transmit_mess(welcome_start_e5402);
		break;

	  case TE5409_DEV_ID:
	        uart_transmit_mess(welcome_start_e5409);
		break;

	  case TE5410_DEV_ID:
	        uart_transmit_mess(welcome_start_e5410);
		break;

	  case TE5416_DEV_ID:
	        uart_transmit_mess(welcome_start_e5416);
		break;

	  default:
	        uart_transmit_mess(welcome_start_exxx);
		break;
	}

        uart_transmit_mess(welcome_end);



	// now prepare for running continuous D-RAM test with
	// results display via UART-A



	// configure D-RAM START/END PTR* and number of DRAM PAGES upon the DSP type
	if(TE5402_BOARD)
	{
		start=((unsigned *) TE5402_DRAM_PAGE_SADDR);
		dram_pages = TE5402_DRAM_PAGES;
	}
     	else
	{
		start=((unsigned *) TE54X_NOTE5402_DRAM_PAGE_SADDR);
		dram_pages = TE54X_NOTE5402_DRAM_PAGES;
	}
	end	= (unsigned *)0xff7f;


        iv = 0x3e91;

        while (1)
	 {


		switch(mem_test(iv,start,end,dram_pages))
		  {
		    case ML_OK:
		        uart_transmit_mess(ok_mess);
			break;

		    default:
		        uart_transmit_mess(err_mess);
			goto error;
		  }


	iv =    (  ((((iv+1)^0x935f)>>8) & 0x00ff)
		 | ((((iv-1)^0xa75a)<<8) & 0xff00)  
		);


	 }


	// trap here on memory test error
error: 
	while(1);

}



