/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   Control Registers test. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo scans continuously control registers within
        the DSP environment.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
	  - HPI is enabled (for E5402/E5416)
***********************************************************************/


#include "te54x_d.h"
#include "c54x_def.h"



main()
{ unsigned i,d,n;

   TE54X_init_for_xdm_dram();
   C54X_INT_DISABLE;


	// NMI selector can't be tested, since this will result in active interrupt
   n=0;
   TE54X_set_mnmi_sel(TE54X_MIRQ_SEL_OFF);
   if(TE54X_get_mnmi_sel()!=TE54X_MIRQ_SEL_OFF) goto error;


	// test loop. Escape to the error idle loop is performed on any error condition.
   while(1)
    {
	TE54X_set_wdt_reset();

	// test for read-back SYS_STAT register
        n=1;
        d=TE54X_get_sys_stat_rg();
	if(   TE54X_get_bmode()!= TE54X_BMODE_NONE
		// check for HPI_ENABLE=1 for all boads except E5402 and E5416
	   || (!(TE5402_BOARD | TE5416_BOARD) && (TE54X_get_hpi_enable())!= TE54X_HPI_ENABLE_ON)
	  ) 
	   goto error;


	// test WDT ENABLE resgister
        n=2;
        TE54X_set_wdt_enable(TE54X_WDT_ENABLE_ON);
	d=TE54X_get_wdt_enable();
	if(d!=TE54X_WDT_ENABLE_ON) break;

        n=3;
        TE54X_set_wdt_enable(TE54X_WDT_ENABLE_OFF);
	d=TE54X_get_wdt_enable();
	if(d!=TE54X_WDT_ENABLE_OFF) break;


	// test DSP MIRQ/NMI selector registers
        for(i=0;i<16;i++)
	 {
	   TE54X_set_mirq0_sel(i);
           TE54X_set_mirq1_sel(~i);
           TE54X_set_mirq2_sel(i+8);
           TE54X_set_mirq3_sel(i+4);

           n=5;
	   d=TE54X_get_mirq0_sel();
	   if(d!=(i&0xf)) goto error;

           n=6;
	   d=TE54X_get_mirq1_sel();
	   if(d!=((~i)&0xf)) goto error;

           n=7;
	   d=TE54X_get_mirq2_sel();
	   if(d!=((i+8)&0xf)) goto error;

           n=8;
	   d=TE54X_get_mirq3_sel();
	   if(d!=((i+4)&0xf)) goto error;
	 }

	TE54X_set_mirq0_sel(TE54X_MIRQ_SEL_OFF);
        TE54X_set_mirq1_sel(TE54X_MIRQ_SEL_OFF);
        TE54X_set_mirq2_sel(TE54X_MIRQ_SEL_OFF);
        TE54X_set_mirq3_sel(TE54X_MIRQ_SEL_OFF);
        TE54X_set_mnmi_sel(TE54X_MIRQ_SEL_OFF);


	// test PIOX/SIOX reset control	
	n=10;
        TE54X_uset_px_reset(TE54X_PXSX_RESET_OFF);
	if(TE54X_uget_px_reset()!=TE54X_PXSX_RESET_OFF) goto error;
        TE54X_uset_px_reset(TE54X_PXSX_RESET_ON);
	if(TE54X_uget_px_reset()!=TE54X_PXSX_RESET_ON) goto error;

	n=11;
        TE54X_uset_sxa_reset(TE54X_PXSX_RESET_OFF);
	if(TE54X_uget_sxa_reset()!=TE54X_PXSX_RESET_OFF) goto error;
        TE54X_uset_sxa_reset(TE54X_PXSX_RESET_ON);
	if(TE54X_uget_sxa_reset()!=TE54X_PXSX_RESET_ON) goto error;

	// test XIOX-B reset for all boards except for E5402
	if(!TE5402_BOARD)
	{
		n=12;
        	TE54X_uset_sxb_reset(TE54X_PXSX_RESET_OFF);
		if(TE54X_uget_sxb_reset()!=TE54X_PXSX_RESET_OFF) goto error;
        	TE54X_uset_sxb_reset(TE54X_PXSX_RESET_ON);
		if(TE54X_uget_sxb_reset()!=TE54X_PXSX_RESET_ON) goto error;
	}

	n=13;
        TE54X_set_pxsx_reset_rg(0);
	if(TE54X_get_pxsx_reset_rg()!=0) goto error;
        TE54X_set_pxsx_reset_rg((!TE5402_BOARD ? 7:3));
	if(TE54X_get_pxsx_reset_rg()!=(!TE5402_BOARD ? 7:3)) goto error;
        TE54X_set_pxsx_reset_rg(0);
	if(TE54X_get_pxsx_reset_rg()!=0) goto error;


	// test PIOX/SIOX TM/XIO-0/1 pins, which are confirgured as OUTPUTs
		// initially configure TM/XIO-0/1 pins as outputs
	n=20;
	TE54X_uset_xio0_cnf(TE54X_XIO_CNF_IO);
	TE54X_uset_xio1_cnf(TE54X_XIO_CNF_IO);
	if(TE54X_get_xio_cnf_rg()!=(TE54X_XIO0_CNF_IO | TE54X_XIO1_CNF_IO)) goto error;

	n=21;
	TE54X_uset_xio0_dir(TE54X_XIO_DIR_OUTPUT);
	TE54X_uset_xio1_dir(TE54X_XIO_DIR_OUTPUT);
	if(TE54X_get_xio_dir_rg()!=(TE54X_XIO0_DIR_OUTPUT | TE54X_XIO1_DIR_OUTPUT)) goto error;

	n=22;
	TE54X_uset_xio0_data(TE54X_XIO_DATA_ON);
	if(TE54X_uget_xio0_data()!=TE54X_XIO_DATA_ON) goto error;
	TE54X_uset_xio0_data(TE54X_XIO_DATA_OFF);
	if(TE54X_uget_xio0_data()!=TE54X_XIO_DATA_OFF) goto error;

	n=23;
	TE54X_uset_xio1_data(TE54X_XIO_DATA_ON);
	if(TE54X_uget_xio1_data()!=TE54X_XIO_DATA_ON) goto error;
	TE54X_uset_xio1_data(TE54X_XIO_DATA_OFF);
	if(TE54X_uget_xio1_data()!=TE54X_XIO_DATA_OFF) goto error;


		// now test DIRECTION control and CNF control registers
	n=24;
	TE54X_uset_xio0_dir(TE54X_XIO_DIR_INPUT);
	if(TE54X_get_xio_dir_rg()!=(TE54X_XIO0_DIR_INPUT | TE54X_XIO1_DIR_OUTPUT)) goto error;

	n=25;
	TE54X_uset_xio1_dir(TE54X_XIO_DIR_INPUT);
	if(TE54X_get_xio_dir_rg()!=(TE54X_XIO0_DIR_INPUT | TE54X_XIO1_DIR_INPUT)) goto error;

	n=26;
	TE54X_uset_xio0_cnf(TE54X_XIO_CNF_TIMER);
	if(TE54X_get_xio_cnf_rg()!=(TE54X_XIO0_CNF_TIMER | TE54X_XIO1_CNF_IO)) goto error;

			// test CNF control for TM/XIO-1 for E5402 board only
	if(TE5402_BOARD)
	{
		n=27;
		TE54X_uset_xio1_cnf(TE54X_XIO_CNF_TIMER);
		if(TE54X_get_xio_cnf_rg()!=(TE54X_XIO0_CNF_TIMER | TE54X_XIO1_CNF_TIMER)) goto error;
	}

	
	// test XDMP register
	n=30;
	TE54X_set_xdm_sel(TE54X_XDM_SEL_DRAM);
	TE54X_set_xdm_page(0);
	if(TE54X_get_xdmp_rg()!=0) goto error;
	
	n=31;
	TE54X_set_xdm_sel(TE54X_XDM_SEL_FLASH);
	TE54X_set_xdm_page(0x15);
	if(TE54X_get_xdmp_rg()!=(TE54X_XDM_SEL_FLASH | 0x15)) goto error;

	n=32;
	TE54X_set_xdm_page(0x0a);
	if(TE54X_get_xdmp_rg()!=(TE54X_XDM_SEL_FLASH | 0xa)) goto error;

	n=33;
	TE54X_set_xdm_page(0xf);
	if(TE54X_get_xdmp_rg()!=(TE54X_XDM_SEL_FLASH | 0xf)) goto error;

	n=34;
	TE54X_set_xdm_sel(TE54X_XDM_SEL_DRAM);
	if(TE54X_get_xdmp_rg()!=(TE54X_XDM_SEL_DRAM | 0xf)) goto error;

	n=35;
	TE54X_set_xdmp_rg(0);
	if(TE54X_get_xdmp_rg()!=0) goto error;

	//finally, test BRS rg
	n=40;
	if(   ((TE548_BOARD | TE549_BOARD) && (TE54X_BRS_RG & 0xff)!=0x80)
	   || ((TE5402_BOARD | TE5409_BOARD | TE5410_BOARD | TE5416_BOARD) &&  (TE54X_BRS_RG & 0xff)!=0x00)
	  ) goto error;
	
    }

	


error:
  while(1);

}



