/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   BSP/BSP1/TDM port test for E548 and E549 boards. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests DSP BSP/BSP1/TDM port for E548/E549. 

	External BSP/BSP1/TDM port transmitter-to-receiver pins shorting
	jumpers are required in order to run this test. These jumpers shall
	connect DX/CLKX/FSX pins to DR/CLKR/FSR pins correspondingly for 
	each of BSP/BSP1/TDM ports. These jumpers shall install into SIOX-A 
	rev.B site header (BSP/BSP1 ports) and SIOX-B rev.C site header 
	(TDM port) on E54x board. Refer to TORNADO-E54x User's Guide for 
	more details about on-board SIOX site headers.

	This test continuously transmits data over BSP/BSP1/TDM ports 
	and verifies the received data against transmitted data.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
	demo-specific jumpers for BSP/BSP1/TDM ports:
	  - jumpers shall connect DX/CLKX/FSX pins to DR/CLKR/FSR pins 
	    correspondingly for each of BSP/BSP1/TDM ports
	  - jumpers shall install into SIOX-A rev.B site header 
	    (BSP/BSP1 ports) and SIOX-B rev.C site header (TDM port)
	    on E54x board. 
***********************************************************************/

#include "c54x_def.h"
#include "te54x_d.h"


#define	WAIT_BSP_XRDY	while(((*C54X_BSP_SPC)&0x800)==0)
#define	WAIT_BSP_RRDY	while(((*C54X_BSP_SPC)&0x400)==0)
#define	WAIT_BSP1_XRDY	while(((*C54X_BSP1_SPC)&0x800)==0)
#define	WAIT_BSP1_RRDY	while(((*C54X_BSP1_SPC)&0x400)==0)
#define	WAIT_TDM_XRDY	while(((*C54X_TDM_TSPC)&0x800)==0)
#define	WAIT_TDM_RRDY	while(((*C54X_TDM_TSPC)&0x400)==0)



fq()
{ unsigned i;
   for(i=0;i<1000;i++);
}

main()
{ unsigned x,r;

   TE54X_init_for_xdm_dram();

	// configure BSP port
	// FSX=high, CLKX=high=1/4*CLKOUT
   *C54X_BSP_SPC=0x0038;	
   *C54X_BSP_SPCE=0x0003;	
   *C54X_BSP_SPC=0x00F8;

	// configure BSP1 port
	// FSX=high, CLKX=high=1/4*CLKOUT
   *C54X_BSP1_SPC=0x0038;
   *C54X_BSP1_SPCE=0x0003;	
   *C54X_BSP1_SPC=0x00F8;	

	// configure TDM port
   *C54X_TDM_TSPC=0x0038;	
   *C54X_TDM_TSPC=0x00F8;

   x=1;

   while(1)
    {
       fq();

	// test BSP port
	// transmit data, verify received data
       WAIT_BSP_XRDY;
       *C54X_BSP_DXR=x;

       WAIT_BSP_RRDY;
       r=*C54X_BSP_DRR;
       if(r!=x) goto error;


	// test BSP1 port
	// transmit data, verify received data
       WAIT_BSP1_XRDY;
       *C54X_BSP1_DXR=x;

       WAIT_BSP1_RRDY;
       r=*C54X_BSP1_DRR;
       if(r!=x) goto error;


	// test TDM port
	// transmit data, verify received data
       WAIT_TDM_XRDY;
       *C54X_TDM_TDXR=x;

       WAIT_TDM_RRDY;
       r=*C54X_TDM_TRCV;
       if(r!=x) goto error;


	// generate next data to be transmitted
       if(x==0x8000) x=1;
	 else x<<1;

    }

	// get here on any verification error
error:
	while(1);

}



