/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   McBSP-0/1/2 ports test for E5402/E5409/E5410/E5416 boards. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests DSP McBSP-0/1/2 ports of E5402/E5409/E5410/E5416. 

	External DX-to-DR shorting jumpers are equired for each of the
	McBSP-0/1/2 ports in order to run this demo. These shorting 
	jumpers shall be connected to SIOX-A rev.B site (McBSP-0/1 ports)
	and SIOX-B rev.C site (McBSP-2 port). Refer to TORNADO-E54x 
	User's Guide for more details about SIOX site headers.

	This test configures all McBSP pins as OUTPUT pins except for
	DR pin, which is configured as INPUT, and continuously generates 
	'running '1'' over DX/DR, FSX, FSR, CLKX and CLKR pins
	for each McBSP port. Oscilloscope can be used to view pulses at
	pins of McBSP-0/1/2 ports.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
	demo-specific jumpers for McBSP ports:
	  - DX-to-DR shorting jumpers are equired for each of the
	    McBSP-0/1/2 ports, which shall be connected to SIOX-A 
	    rev.B site (McBSP-0/1 ports) and SIOX-B rev.C site 
	    (McBSP-2 port). 
***********************************************************************/

#include "c54x_def.h"
#include "te54x_d.h"


#define DX_OUT_0      0x3f00
#define DX_OUT_1      0x3f20
#define FSX_OUT_0     0x3f00
#define FSX_OUT_1     0x3f08
#define FSR_OUT_0     0x3f00
#define FSR_OUT_1     0x3f04
#define CLKX_OUT_0    0x3f00
#define CLKX_OUT_1    0x3f02
#define CLKR_OUT_0    0x3f00
#define CLKR_OUT_1    0x3f01

#define DR_BIT_MASK   0x10
#define CLKS_BIT_MASK 0x40
#define CLKR_BIT_MASK 0x1
#define CLKX_BIT_MASK 0x2
#define FSX_BIT_MASK  0x8
#define FSR_BIT_MASK  0x4



unsigned pass_counter=0;

main()
{
   	TE54X_init_for_xdm_dram();

	// configure all McBSP-0 pins as OUTPUTS except for DR pin,
	// whcih is configured as INPUT
   	*C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR1;
   	*C54X_MCBSP0_SPCR1 = 0;
   	*C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR2;
   	*C54X_MCBSP0_SPCR2 = 0;

	// configure all McBSP-1 pins as OUTPUTS except for DR pin,
	// whcih is configured as INPUT
   	*C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_SPCR1;
   	*C54X_MCBSP1_SPCR1 = 0;
   	*C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_SPCR2;
   	*C54X_MCBSP1_SPCR2 = 0;

   	if(!TE5402_BOARD)
   	{
		// configure all McBSP-2 pins as OUTPUTS except for DR pin,
		// whcih is configured as INPUT
   		*C54X_MCBSP2_SPSA = C54X_MCBSP_SBA_SPCR1;
   		*C54X_MCBSP2_SPCR1 = 0;
   		*C54X_MCBSP2_SPSA = C54X_MCBSP_SBA_SPCR2;
   		*C54X_MCBSP2_SPCR2 = 0;
   	}


   	while(1)
    {
      // test McBSP-0 port
      *C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_PCR;

		// set DX=1
      *C54X_MCBSP0_PCR = DX_OUT_1;
      if(((*C54X_MCBSP0_PCR) & DR_BIT_MASK  ) ==0) break;

		// set DX=0
      *C54X_MCBSP0_PCR = DX_OUT_0;
      if((*C54X_MCBSP0_PCR & DR_BIT_MASK  ) !=0) break;

		// generate pulse at FSX
      *C54X_MCBSP0_PCR = FSX_OUT_1;
      *C54X_MCBSP0_PCR = FSX_OUT_0;

		// generate pulse at FSR
      *C54X_MCBSP0_PCR = FSR_OUT_1;
      *C54X_MCBSP0_PCR = FSR_OUT_0;

		// generate pulse at CLKX
      *C54X_MCBSP0_PCR = CLKX_OUT_1;
      *C54X_MCBSP0_PCR = CLKX_OUT_0;

		// generate pulse at CLKR
      *C54X_MCBSP0_PCR = CLKR_OUT_1;
      *C54X_MCBSP0_PCR = CLKR_OUT_0;



      // test McBSP-1 port
      *C54X_MCBSP1_SPSA = C54X_MCBSP_SBA_PCR;

		// set DX=1
      *C54X_MCBSP1_PCR = DX_OUT_1;
      if(((*C54X_MCBSP1_PCR) & DR_BIT_MASK  ) ==0) break;

		// set DX=0
      *C54X_MCBSP1_PCR = DX_OUT_0;
      if((*C54X_MCBSP1_PCR & DR_BIT_MASK  ) !=0) break;

		// generate pulse at FSX
      *C54X_MCBSP1_PCR = FSX_OUT_1;
      *C54X_MCBSP1_PCR = FSX_OUT_0;

		// generate pulse at FSR
      *C54X_MCBSP1_PCR = FSR_OUT_1;
      *C54X_MCBSP1_PCR = FSR_OUT_0;

		// generate pulse at CLKX
      *C54X_MCBSP1_PCR = CLKX_OUT_1;
      *C54X_MCBSP1_PCR = CLKX_OUT_0;

		// generate pulse at CLKR
      *C54X_MCBSP1_PCR = CLKR_OUT_1;
      *C54X_MCBSP1_PCR = CLKR_OUT_0;


	  // skip McBSP-2 port test for E5402 board.
	  if(TE5402_BOARD)	continue;

      // test McBSP-2 port
      *C54X_MCBSP2_SPSA = C54X_MCBSP_SBA_PCR;

		// set DX=1
      *C54X_MCBSP2_PCR = DX_OUT_1;
      if(((*C54X_MCBSP2_PCR) & DR_BIT_MASK  ) ==0) break;

		// set DX=0
      *C54X_MCBSP2_PCR = DX_OUT_0;
      if((*C54X_MCBSP2_PCR & DR_BIT_MASK  ) !=0) break;

		// generate pulse at FSX
      *C54X_MCBSP2_PCR = FSX_OUT_1;
      *C54X_MCBSP2_PCR = FSX_OUT_0;

		// generate pulse at FSR
      *C54X_MCBSP2_PCR = FSR_OUT_1;
      *C54X_MCBSP2_PCR = FSR_OUT_0;

		// generate pulse at CLKX
      *C54X_MCBSP2_PCR = CLKX_OUT_1;
      *C54X_MCBSP2_PCR = CLKX_OUT_0;

		// generate pulse at CLKR
      *C54X_MCBSP2_PCR = CLKR_OUT_1;
      *C54X_MCBSP2_PCR = CLKR_OUT_0;

     	pass_counter++;
	}

error:
	while(1);

}



