/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   USART channels A/B test in ASYNC mode. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests channels A and B of USART in ASYNC mode.
	USART channel B is tested with RS232C external interface,
	whereas USART channel A is tested with EIA530 (RS422)
	external interface.

	External 9600 baud ASYNC terminal with RS232C interface or 
	PC COM port running in terminal mode at 9600 bauds shall be 
	used with this demo, which must be connected to RS232 interface
	for USART channel B. Echo plug must be installed into
	EIA530 (RS422) connector for USART channel A, which 
	returns transmitted data back to the receiver input.

	After transmission of greeting message to external terminal,
	demo enters echo mode, which performs transmission of 
	received character back to terminal.

	The following is the data transmission scheme for this demo:
	      PC -> RxB/RS232 -s/w-> TxA/RS422 -> RxA/RS422 -s/w-> TxB/RS232 -> PC


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
	on-board SW4 switch:
	  - RS232C I/F must be used as external I/F for USART channel-B
	  - EIA530/RS422 I/F with TCLK-INT-OUT (internally generated 
	    transmitter clock, which is output to connector), must be 
	    used as external I/F for USART channel-A
	  - EIA530/RS422 echo plug must be installed into EIA530/RS422 I/F
	    connector for USART channel A, which return transmitted
	    data to received nput, transmitter clock to receiver clock, and
	    RTS to the CTS.
  	on-board socket S2:
	  - 1.8432 MHz crystal oscillator must be used as USART
	    source/master clock
***********************************************************************/
#include	"te54x_d.h"

char welcome_start_e548[]	= "TORNADO-E548";
char welcome_start_e549[]	= "TORNADO-E549";
char welcome_start_e5402[]	= "TORNADO-E5402";
char welcome_start_e5409[]	= "TORNADO-E5409";
char welcome_start_e5410[]	= "TORNADO-E5410";
char welcome_start_e5416[]	= "TORNADO-E5416";
char welcome_start_exxx[]	= "TORNADO-E54??? ";
char welcome_end[]		= " USART B->A->B test\n\n>>";


usart_init()
{
	// configure USART PIO port as outputs
	TE54X_set_usart_rg(TE54X_USART_A_PCR_RG,0);


	// now configure USART channel-A in ASYNC mode with 9600 baud rate
        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,0x0);
	TE54X_set_usart_rg(TE54X_USART_A_MODE_RG,0x0);

        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,0x43); // MCE, ASYNC
	TE54X_set_usart_rg(TE54X_USART_A_CCR1_RG,0x10); // ODS, BCR, clk_mode=7
	TE54X_set_usart_rg(TE54X_USART_A_CCR2_RG,0x38); // BDF, SSEL (clk_mode=7b), TCLK = out
	TE54X_set_usart_rg(TE54X_USART_A_DAFO_RG,0x00); // STOP=1, PARity=OFF, 8bit 
	TE54X_set_usart_rg(TE54X_USART_A_RFC_RG, 0x00); // RFIFO threshold = 1 
        TE54X_set_usart_rg(TE54X_USART_A_BGR_RG, 95);	// 9600 bauds via 1,8432 MHz crystal oscillator

        TE54X_set_usart_rg(TE54X_USART_A_CCR0_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_CCR0_RG)| 0x80
			  );				// PU: set channel activity

	TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x41); // receiver/transmitter reset
	TE54X_set_usart_rg(TE54X_USART_A_MODE_RG,0x0c);	// receiver active, set RTS 

	TE54X_set_usart_rg(TE54X_USART_A_IPC_RG, 0x81);	// set unmasked irqs visible, irq active low 


	// now configure USART channel-B in ASYNC mode with 9600 baud rate
        TE54X_set_usart_rg(TE54X_USART_B_CCR0_RG,0x0);
	TE54X_set_usart_rg(TE54X_USART_B_MODE_RG,0x0);

        TE54X_set_usart_rg(TE54X_USART_B_CCR0_RG,0x43); // MCE, ASYNC
	TE54X_set_usart_rg(TE54X_USART_B_CCR1_RG,0x1f); // ODS, BCR, clk_mode=7
	TE54X_set_usart_rg(TE54X_USART_B_CCR2_RG,0x38); // BDF, SSEL (clk_mode=7b), TCLK = out
	TE54X_set_usart_rg(TE54X_USART_B_DAFO_RG,0x00); // STOP=1, PARity=OFF, 8bit 
	TE54X_set_usart_rg(TE54X_USART_B_RFC_RG, 0x00); // RFIFO threshold = 1 
        TE54X_set_usart_rg(TE54X_USART_B_BGR_RG, 0x05);	// 9600 bauds via 1,8432 MHz crystal oscillator

        TE54X_set_usart_rg(TE54X_USART_B_CCR0_RG,
			   TE54X_get_usart_rg(TE54X_USART_B_CCR0_RG)| 0x80
			  );				// PU: set channel activity

	TE54X_set_usart_rg(TE54X_USART_B_CMDR_RG,0x41); // receiver/transmitter reset
	TE54X_set_usart_rg(TE54X_USART_B_MODE_RG,0x0c);	// receiver active, set RTS 
}


#define WAIT_CMDR_READY_A   {while (TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) &0x4);}

uart_transmit_char_A(c)
int c;
{
	// transmit using XPR irq with bits ALLS/XPR from ISR1 
  while (!(TE54X_get_usart_rg(TE54X_USART_A_ISR1_RG) &0x21));
  TE54X_set_usart_rg(TE54X_USART_A_TFIFO_RG,c);
  WAIT_CMDR_READY_A;
  TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x8);

/* transmission using TIC: runs well always
  while ((TE54X_get_usart_rg(TE54X_USART_A_STAR_RG)&0x8));
  TE54X_set_usart_rg(TE54X_USART_A_TIC_RG,c);
*/
}


int uart_receive_char_A()
{ int c;

  while (!(TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) &0x20));
  c= TE54X_get_usart_rg(TE54X_USART_A_RFIFO_RG);
  WAIT_CMDR_READY_A;
  TE54X_set_usart_rg(TE54X_USART_A_CMDR_RG,0x80);	// release RFIFO
  return(c);
}



int uart_receiver_status_A()
{
  return((TE54X_get_usart_rg(TE54X_USART_A_STAR_RG) & 0x20) ? 1:0);
}


#define WAIT_CMDR_READY_B   {while (TE54X_get_usart_rg(TE54X_USART_B_STAR_RG) &0x4);}

uart_transmit_char_B(c)
int c;
{
	// transmit using XPR irq with bits ALLS/XPR from ISR1 
  while (!(TE54X_get_usart_rg(TE54X_USART_B_ISR1_RG) &0x21));
  TE54X_set_usart_rg(TE54X_USART_B_TFIFO_RG,c);
  WAIT_CMDR_READY_B;
  TE54X_set_usart_rg(TE54X_USART_B_CMDR_RG,0x8);

/* transmission using TIC: runs well always
  while ((TE54X_get_usart_rg(TE54X_USART_B_STAR_RG)&0x8));
  TE54X_set_usart_rg(TE54X_USART_B_TIC_RG,c);
*/
}


int uart_receive_char_B()
{ int c;

  while (!(TE54X_get_usart_rg(TE54X_USART_B_STAR_RG) &0x20));
  c= TE54X_get_usart_rg(TE54X_USART_B_RFIFO_RG);
  WAIT_CMDR_READY_B;
  TE54X_set_usart_rg(TE54X_USART_B_CMDR_RG,0x80);	// release RFIFO
  return(c);
}


int uart_receiver_status_B()
{
  return((TE54X_get_usart_rg(TE54X_USART_B_STAR_RG) & 0x20) ? 1:0);
}


uart_transmit_mess_B(s)
char *s;
{
  while((*s &0xff)!=0) uart_transmit_char_B(*(s++));
}


void main()
{
	// initialize DSP for running with SRAM as D-MEM
	TE54X_init_for_xdm_dram();

	// init USART
	usart_init();

	// transmit greeting message
	switch(TE54X_get_dev_id())
	{
	  case TE548_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e548);
		break;

	  case TE549_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e549);
		break;

	  case TE5402_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e5402);
		break;

	  case TE5409_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e5409);
		break;

	  case TE5410_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e5410);
		break;

	  case TE5416_DEV_ID:
	        uart_transmit_mess_B(welcome_start_e5416);
		break;

	  default:
	        uart_transmit_mess_B(welcome_start_exxx);
		break;
	}

        uart_transmit_mess_B(welcome_end);

	// enter echo loop
        while (1)
	 {
	   if(uart_receiver_status_B()==1)
              uart_transmit_char_A(uart_receive_char_B());

	   if(uart_receiver_status_A()==1)
              uart_transmit_char_B(uart_receive_char_A());
	 }
}



