/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   USB controller test. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo scans registers of USB controller, address lines,
	and generates USB interrupts.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
***********************************************************************/


#include	"te54x_d.h"


// USB write-to-read pause, which is required for
// operations as WRITE to SCR register, which initializes
// internal state machine operations and requires some
// time to execute
// E54x DSP is very fast and immediate reads may return 
// incorrect results.
wait()
{ int i;
   for (i=0;i<5;i++);
}



main()
{ int i, pass;
  unsigned d;

   // initialize DSP for running with SRAM as D-MEM
   TE54X_init_for_xdm_dram();

   pass=0;
   while(1)
    {
	// issue WDT reset signal in the beginning of each cycle, which
	// can be used as SYNC for oscilloscope
	TE54X_set_wdt_reset();

		//-- check USB data bus via running 1<< to the SBIE reg
        for(i=0,d=1;i<=8;i++)
	 {
           TE54X_set_usb_rg(TE54X_USB_SBIE_RG, i==0 ? 0:d);	
           TE54X_set_usb_rg(TE54X_USB_SBIE1_RG, 0xff);	// perform false bus data

           if(TE54X_get_usb_rg(TE54X_USB_SBIE_RG) != (i==0 ? 0:d))
		goto error;	

           if(i>0)
	      {	d=(d<<1);
	      }
	 }

		//--- perform software reset of USB controller
		//    and verify the results of reset
           TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x8);	
	wait();
           TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0);	
	wait();
           if(   TE54X_get_usb_rg(TE54X_USB_TXCNTH_RG)!=0
	      || TE54X_get_usb_rg(TE54X_USB_TXCNTL_RG)!=0
              || TE54X_get_usb_rg(TE54X_USB_SBIE_RG)!=0	
              || TE54X_get_usb_rg(TE54X_USB_SBIE1_RG)!=0	
              || TE54X_get_usb_rg(TE54X_USB_SCR_RG)!=0
             ) break;	


		//--- now toggle USB interrupt line using INT polarity bit
           TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x80);	
	wait();
           if(TE54X_get_usb_rg(TE54X_USB_SCR_RG)!=0x80) break;	
           TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x00);	
	wait();
           if(TE54X_get_usb_rg(TE54X_USB_SCR_RG)!=0x00) break;	


         	//--- now write random data to different USB registers
           TE54X_set_usb_rg(TE54X_USB_TXCNTH_RG, 0);	
           TE54X_set_usb_rg(TE54X_USB_TXCNTL_RG, 0x91);	
           TE54X_set_usb_rg(TE54X_USB_SBIE_RG, 0x5a);	
           TE54X_set_usb_rg(TE54X_USB_SBIE1_RG, 0xa5);	
           TE54X_set_usb_rg(TE54X_USB_FADDR_RG, 0x47);	
           TE54X_set_usb_rg(TE54X_USB_SCRATCH_RG, 0x6e);	
           TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x92);	

           if( (pass%2) ==0)
		   TE54X_set_usb_rg(TE54X_USB_MCSR_RG, 0x0);
	     else  TE54X_set_usb_rg(TE54X_USB_MCSR_RG, 0x1);


           if(TE54X_get_usb_rg(TE54X_USB_TXCNTL_RG)!=0x91) break;	
           if(TE54X_get_usb_rg(TE54X_USB_SBIE_RG)!=0x5a) break;	
           if(TE54X_get_usb_rg(TE54X_USB_SBIE1_RG)!=0xa5) break;	
           if(TE54X_get_usb_rg(TE54X_USB_FADDR_RG)!=0x47) break;	
           if(TE54X_get_usb_rg(TE54X_USB_SCRATCH_RG)!=0x6e) break;	
           if(TE54X_get_usb_rg(TE54X_USB_SCR_RG)!=0x92) break;	

           pass++;
	   if(pass>1000) pass=0;

    }


  error:
    while(1) ;

}



