/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   USART/USB interrupt request test. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests USART and USB interrupt requests. 

	USART and USB interrupt requests are continuously simulated via 
	DSP software using continuous toggling of interrupt polarity via 
	the corresponding USART/USB control registers (refer to USART and 
	USB documentation for more details).

	User can observe DSP response to USART/USB interrupt requests via 
	USART PIO-0/1 I/O pins,	which are configued as the OUTPUTS. PIO-0/1 
	outputs can be observed via oscilloscope.
	
	IMPORTANT NOTE:
		This test does not perform testing of SIOX/PIOX
		interrupt requests.

	External interrupt request selectors for C54x DSP and the 
	corresponding USART PIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:	DSP IRQ:	USART PIO output:
		-----------	--------	-----------------
		USART IRQ	INT0		PIO-0
		USB IRQ		INT1		PIO-1

   TORNADO platform:
   -----------------
	MIRAGE JTAG emulator and TI CCS tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
***********************************************************************/

#include	"te54x_d.h"
#include 	"c54x_def.h"


interrupt void c_xint0()
// invert PIO-0 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x1
			  ); 
}

interrupt void c_xint1()
// invert PIO-1 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x2
			  ); 
}


// USB write-to-read pause, which is required for
// operations as WRITE to SCR register, which initializes
// internal state machine operations and requires some
// time to execute
// E54x DSP is very fast and immediate reads may return 
// incorrect results.
_wait_usb()
{ int i;
   for (i=0;i<5;i++);
}



main()
{ unsigned j;

	// initialize DSP for running with SRAM as D-MEM
	TE54X_init_for_xdm_dram();

	// configure USART PIO port as outputs and clear outputs
	TE54X_set_usart_rg(TE54X_USART_A_PCR_RG,0);
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,0);

	// reset USART IMR
	TE54X_set_usart_rg(TE54X_USART_A_IMR0_RG,0xff);
	TE54X_set_usart_rg(TE54X_USART_A_IMR1_RG,0xff);
	TE54X_set_usart_rg(TE54X_USART_B_IMR0_RG,0xff);
	TE54X_set_usart_rg(TE54X_USART_B_IMR1_RG,0xff);
	TE54X_set_usart_rg(TE54X_USART_A_IPC_RG, 0x01);	// set IRQ PUSH/PULL, active low
	

	// configure E54x DSP XIRQ selectors
	TE54X_set_mirq0_sel(TE54X_MIRQ_SEL_USART);
	TE54X_set_mirq1_sel(TE54X_MIRQ_SEL_USB);

	// perform software reset of USB controller
        TE54X_set_usb_rg(TE54X_USB_SBIE_RG, 0);	
        TE54X_set_usb_rg(TE54X_USB_SBIE1_RG,0);
        TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x8);	
	_wait_usb();


	// enable C54x external interrupt inputs INT-0..3
	C54X_XINT0_ENABLE;
	C54X_XINT1_ENABLE;

	// enable external IRQs
	C54X_INT_ENABLE;

	
	// loop forever
	j=0;
	while(1)
	{

	  // generate USART IRQ on ever 100-th cycle
	  if((j%30)==0)
	  {
		// toggle USART interrupt line using INT polarity bit
		TE54X_set_usart_rg(TE54X_USART_A_IPC_RG, 0x03);	
		TE54X_set_usart_rg(TE54X_USART_A_IPC_RG, 0x01);	
	  }


	  // generate USB IRQ on ever 100-th cycle
	  if((j%100)==0)
	  {
		// toggle USB interrupt line using INT polarity bit
		TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x80);	
		_wait_usb();
		TE54X_set_usb_rg(TE54X_USB_SCR_RG, 0x00);	
		_wait_usb();
	  }

	  if(++j >=1000) j=0;
	}
}


