/********************************************************************
   TORNADO-E54x DCM Demo Utility.
   SIOX/PIOX IRQ test (SX_IRQ-0/1/2 and PX_IRQ-0/1). rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests SIOX/PIOX interrupt requests (XIRQs). XIRQs for
	E54x comprise of SIOX-A/B interrupt requests (SX_IRQ-0/1/2) and
	of PIOX interrupt requests (PX_IRQ-0/1). User can apply external
	SIOX/PIOX interrupt request from signal generator, etc sources
	and observe DSP response via USART PIO-0..4 I/O pins, which are 
	configued as the OUTPUTS. PIO-0..4 outputs can be observed via 
	oscilloscope.
	
	IMPORTANT NOTE:
		This test does not perform testing of USART and USB
		interrupt requests.

	SIOX/PIOX interrupt request selectors for C54x DSP and the 
	corresponding USART PIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:	DSP IRQ:	USART PIO output:
		-----------	--------	-----------------
		SX_IRQ-0	INT0		PIO-0
		SX_IRQ-1	INT1		PIO-1
		SX_IRQ-2	INT2		PIO-2
		PX_IRQ-0	INT3		PIO-3
		PX_IRQ-1	NMI		PIO-4

   TORNADO platform:
   -----------------
	MIRAGE JTAG emulator and TI CCS tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - no BOOT mode
***********************************************************************/

#include	"te54x_d.h"
#include 	"c54x_def.h"


interrupt void c_xint0()
// invert PIO-0 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x1
			  ); 
}

interrupt void c_xint1()
// invert PIO-1 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x2
			  ); 
}

interrupt void c_xint2()
// invert PIO-2 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x4
			  ); 
}


interrupt void c_xint3()
// invert PIO-3 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x8
			  ); 
}


interrupt void c_xnmi()
// invert PIO-4 output state
{
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,
			   TE54X_get_usart_rg(TE54X_USART_A_PVR_RG) ^ 0x10
			  ); 
}


main()
{
	// initialize DSP for running with SRAM as D-MEM
	TE54X_init_for_xdm_dram();

	// configure USART PIO port as outputs and clear outputs
	TE54X_set_usart_rg(TE54X_USART_A_PCR_RG,0);
	TE54X_set_usart_rg(TE54X_USART_A_PVR_RG,0);

	// configure E54x DSP XIRQ selectors
	TE54X_set_mirq0_sel(TE54X_MIRQ_SEL_SXIRQ0);
	TE54X_set_mirq1_sel(TE54X_MIRQ_SEL_SXIRQ1);
	TE54X_set_mirq2_sel(TE54X_MIRQ_SEL_SXIRQ2);
	TE54X_set_mirq3_sel(TE54X_MIRQ_SEL_PXIRQ0);
	TE54X_set_mnmi_sel(TE54X_MIRQ_SEL_PXIRQ1);

	// enable C54x external interrupt inputs INT-0..3
	C54X_XINT0_ENABLE;
	C54X_XINT1_ENABLE;
	C54X_XINT2_ENABLE;
	C54X_XINT3_ENABLE;

	// enable external IRQs
	C54X_INT_ENABLE;

	// loop forever
	while(1);
}


