/***************************************************************
  TORNADO-E54x DSP utilities. Rev.2A
  (C) MicroLAB Systems Ltd, 1997-2001

  covers:
  -------
  	E548/E549/E5402/E5409/E5410/E5416 rev.2.x

  copyright:
  ----------
	This utility is supplied free of charge as it is without
	any obligation from MicroLAB Systems. No responsibility is 
	assumed	in case of any use or misuse of these utilities.

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com

  notes:
  ------
	1. This file has been designed with TAB setting 8.
****************************************************************/


#ifndef __TE54X_DEF__
#define __TE54X_DEF__		1


#define	ON			1
#define	OFF			0
#define	TE54X_ON		1
#define	TE54X_OFF		0



//===== recommended SWWSR/BSCR/SWCR/CLKMD register settings ======
// RAM on-chip -0ws
// ext P-RAM   - 1ws
// ext SRAM/DM - 1ws 
// ext FLASH/DM- 2ws 
// IO          - 2ws (required for external ready processing)
// CLKOUT      = 1xDSP CLK or 1/2xDSP CLK
// Note !!! Use TE54X_init_dsp() utility function in order to
//          automatically configure DSP wait state control registers.

	// SWWSR register setting in case external D-MEM is SRAM memory
#define TE548_SWWSR_DATA_DRAM		0x2209  // ext P/D-RAM - 1ws, IO - 2ws
#define TE549_SWWSR_DATA_DRAM		0x2209  // ext P/D-RAM - 1ws, IO - 2ws
#define TE5402_SWWSR_DATA_DRAM		0x2249  // ext P/D-RAM - 1ws, IO - 2ws
#define TE5409_SWWSR_DATA_DRAM		0x2209	// ext P/D-RAM - 1ws, IO - 2ws
#define TE5410_SWWSR_DATA_DRAM		0x2209	// ext P/D-RAM - 1ws, IO - 2ws
#define TE5416_SWWSR_DATA_DRAM		0x2209	// ext P/D-RAM - 1ws, IO - 2ws

	// SWWSR register setting in case external D-MEM is FLASH memory
#define TE548_SWWSR_DATA_FLASH		0x2409  // ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws
#define TE549_SWWSR_DATA_FLASH		0x2409  // ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws
#define TE5402_SWWSR_DATA_FLASH		0x2489  // ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws
#define TE5409_SWWSR_DATA_FLASH		0x2409	// ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws
#define TE5410_SWWSR_DATA_FLASH		0x2409	// ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws
#define TE5416_SWWSR_DATA_FLASH		0x2409	// ext P-RAM - 1ws, D-MEM=FLASH - 2ws, IO - 2ws

	// other DSp on-chip register settings
#define TE548_CLKMD_DATA		0xF7f7	// CLKMD reg (PLLx1)
#define TE549_CLKMD_DATA		0xF7f7	// CLKMD reg (PLLx1)
#define TE5402_CLKMD_DATA		0x17f7	// CLKMD reg (PLLx2)
#define TE5409_CLKMD_DATA		0xF7f7	// CLKMD reg (PLLx1)
#define TE5410_CLKMD_DATA		0xF7f7	// CLKMD reg (PLLx1)
#define TE5416_CLKMD_DATA		0x47f7	// CLKMD reg (PLLx5)
#define TE548_BSCR_DATA			0x8800	// BSCR reg (CLKOUT=80MHz) 
#define TE549_BSCR_DATA			0x8800	// BSCR reg (CLKOUT=100MHz)
#define TE5402_BSCR_DATA		0xc800	// BSCR reg (CLKOUT=100MHZ)
#define TE5409_BSCR_DATA		0x8800	// BSCR reg (CLKOUT=80MHz) 
#define TE5410_BSCR_DATA		0x8000	// BSCR reg (CLKOUT=100MHz)
#define TE5416_BSCR_DATA		0xa000	// BSCR reg (CLKOUT=DSPCLK/2=80MHz)
#define TE549_SWCR_DATA			0x0000	// SWCR reg (WSx1)
#define TE5402_SWCR_DATA		0x0000	// SWCR reg (WSx1)
#define TE5409_SWCR_DATA		0x0000	// SWCR reg (WSx1)
#define TE5410_SWCR_DATA		0x0000	// SWCR reg (WSx1)
#define TE5416_SWCR_DATA		0x0000	// SWCR reg (WSx1)

	// Intermediate data for CLKMD rg prior loading final data 
	// (corresponds to CLKIN/2). This is required in order to 
	// configure C54x PLL via software.
#define TE54X_CLKMD_PRESET_DATA		0x07f0	

//================================================================


//=================== E54x Memory Areas ==========================

	//----------- external P-RAM defs ------
#define	TE54X_PRAM_SADDR		0x00000
#define	TE54X_PRAM_EADDR		0x1ffff
#define	TE54X_PRAM_LEN			0x20000
#define	TE54X_PRAM_PAGE_LEN		0x10000
#define	TE54X_PRAM_PAGES		2


	//----------- external D-RAM def ------
	// (use TE54X_set_xdmem_page() function in order to select 
	// external D-RAM and to set D-RAM page)
#define	TE548_DRAM_PAGE_SADDR		0x08000
#define	TE548_DRAM_PAGE_EADDR		0x0ffff
#define	TE548_DRAM_PAGE_LEN		0x08000
#define	TE548_DRAM_PAGES		4

#define	TE549_DRAM_PAGE_SADDR		0x08000
#define	TE549_DRAM_PAGE_EADDR		0x0ffff
#define	TE549_DRAM_PAGE_LEN		0x08000
#define	TE549_DRAM_PAGES		4

#define	TE5402_DRAM_PAGE_SADDR		0x04000
#define	TE5402_DRAM_PAGE_EADDR		0x0ffff
#define	TE5402_DRAM_PAGE_LEN		0x0C000
#define	TE5402_DRAM_PAGES		2

#define	TE5409_DRAM_PAGE_SADDR		0x08000
#define	TE5409_DRAM_PAGE_EADDR		0x0ffff
#define	TE5409_DRAM_PAGE_LEN		0x08000
#define	TE5409_DRAM_PAGES		4

#define	TE5410_DRAM_PAGE_SADDR		0x08000
#define	TE5410_DRAM_PAGE_EADDR		0x0ffff
#define	TE5410_DRAM_PAGE_LEN		0x08000
#define	TE5410_DRAM_PAGES		4

#define	TE5416_DRAM_PAGE_SADDR		0x08000
#define	TE5416_DRAM_PAGE_EADDR		0x0ffff
#define	TE5416_DRAM_PAGE_LEN		0x08000
#define	TE5416_DRAM_PAGES		4
		// general DRAM page info for all not-TE5402 boards
#define	TE54X_NOTE5402_DRAM_PAGE_SADDR	0x08000
#define	TE54X_NOTE5402_DRAM_PAGE_EADDR	0x0ffff
#define	TE54X_NOTE5402_DRAM_PAGE_LEN	0x08000
#define	TE54X_NOTE5402_DRAM_PAGES	4


	//----------- external EPROM/FLASH def ------
	// 1. For low-level direct access use TE54X_set_xdmem_page() in order 
	//    to select FLASH memory and and to set FLASH page.
	// 2. For high-level access to AMD Am29F0x0 FLASH use FLASH programming 
	//    utilities.
#define TE54X_FLASH_PAGE_SADDR		0x8000	
#define TE54X_FLASH_PAGE_EADDR		0xffff
#define TE54X_FLASH_PAGE_LEN		0x8000
#define TE54X_FLASH_DATA_BITMASK	0xff

#define TE54X_FLASH_1M_PAGES		4		// number of pages for FLASH/EPROM 1M (128Kx8)
#define TE54X_FLASH_2M_PAGES		8		// number of pages for FLASH/EPROM 2M (256Kx8)
#define TE54X_FLASH_4M_PAGES		16		// number of pages for FLASH/EPROM 4M (512Kx8)
#define TE54X_FLASH_8M_PAGES		32		// number of pages for FLASH/EPROM 8M (1Mx8)
#define TE54X_FLASH_1M_LEN		0x020000L	// length of FLASH/EPROM 1M (128Kx8)
#define TE54X_FLASH_2M_LEN		0x040000L	// length of FLASH/EPROM 2M (256Kx8)
#define TE54X_FLASH_4M_LEN		0x080000L	// length of FLASH/EPROM 4M (512Kx8)
#define TE54X_FLASH_8M_LEN		0x100000L	// length of FLASH/EPROM 8M (1Mx8)
#define TE54X_FLASH_LEN			0x100000L	// maximum length of FLASH/EPROM

//================================================================


//=================== E54x I/O ports =============================

	//---- 8-bit control registers area
	//     (requires XF=1 during access)
	//     (use acess functions and macros below for access)

#define TE54X_CNTR_DATA_BITMASK		0xff

		// external DRAM/FLASH memory page register
#define TE54X_XDMP_RG			port8000	

		// DSP external IRQ selectors
#define TE54X_MIRQ0_SEL_RG		porta000	
#define TE54X_MIRQ1_SEL_RG		porta001	
#define TE54X_MIRQ2_SEL_RG		porta002	
#define TE54X_MIRQ3_SEL_RG		porta003	
#define TE54X_MNMI_SEL_RG		porta004	

		// system control/status
#define TE54X_DEV_ID_RG			portfffd	// r-only	
#define TE54X_SYS_STAT_RG		portfffe	// r-only
#define TE54X_BRS_RG			portffff	// r-only, contains LSB of BRS, not for user access

		// WDT control control
#define TE54X_WDT_ENABLE_RG		port8007	
#define TE54X_WDT_RESET_RG		port4000	// w-only, data is ignored

		// PIOX/SIOX reset control
#define TE54X_PXSX_RESET_RG		port8006

		// PIOX/SIOX TM/XIO-0/1 timer/IO pins control
#define TE54X_XIO_CNF_RG		port8003
#define TE54X_XIO_DIR_RG		port8002
#define TE54X_XIO_DATA_RG		port8001


	//---- 8-bit Agere/Lucent USS-820/825 USB controller regs 
	//     (require XF=1 during access)
	//     (use access functions below)
#define TE54X_USB_DATA_BITMASK		0xff

#define	TE54X_USB_TXDAT_RG		port2000
#define	TE54X_USB_TXCNTL_RG		port2001
#define	TE54X_USB_TXCNTH_RG		port2002
#define	TE54X_USB_TXCON_RG		port2003
#define	TE54X_USB_TXFLG_RG		port2004
#define	TE54X_USB_RXDAT_RG		port2005
#define	TE54X_USB_RXCNTL_RG		port2006
#define	TE54X_USB_RXCNTH_RG		port2007
#define	TE54X_USB_RXCON_RG		port2008
#define	TE54X_USB_RXFLG_RG		port2009
#define	TE54X_USB_EPINDEX_RG		port200a
#define	TE54X_USB_EPCON_RG		port200b
#define	TE54X_USB_TXSTAT_RG		port200c
#define	TE54X_USB_RXSTAT_RG		port200d
#define	TE54X_USB_SOFL_RG		port200e
#define	TE54X_USB_SOFH_RG		port200f
#define	TE54X_USB_FADDR_RG		port2010
#define	TE54X_USB_SCR_RG		port2011
#define	TE54X_USB_SSR_RG		port2012
#define	TE54X_USB_SBI_RG		port2014
#define	TE54X_USB_SBI1_RG		port2015
#define	TE54X_USB_SBIE_RG		port2016
#define	TE54X_USB_SBIE1_RG		port2017
#define	TE54X_USB_REV_RG		port2018
#define	TE54X_USB_LOCK_RG		port2019
#define	TE54X_USB_PEND_RG		port201A
#define	TE54X_USB_SCRATCH_RG		port201B
#define	TE54X_USB_MCSR_RG		port201C



	//---- 8-bit SAB82532 USART (ESCC2) registers 
	//     (require XF=1 during access)
	//     (use access macros below)
#define TE54X_USART_DATA_BITMASK	0xff

		// USART channel #A
			// registers for HDLC mode
#define	TE54X_USART_A_RFIFO_RG		port0000	// r-only	
#define	TE54X_USART_A_TFIFO_RG		port0000	// w-only
#define	TE54X_USART_A_STAR_RG		port0020	// r-only
#define	TE54X_USART_A_CMDR_RG		port0020	// w-only
#define	TE54X_USART_A_RSTA_RG		port0021	// r-only
#define	TE54X_USART_A_PRE_RG		port0021	// w-only
#define	TE54X_USART_A_MODE_RG		port0022
#define	TE54X_USART_A_TIMR_RG		port0023
#define	TE54X_USART_A_XAD1_RG		port0024
#define	TE54X_USART_A_XAD2_RG		port0025
#define	TE54X_USART_A_RAH1_RG		port0026
#define	TE54X_USART_A_RAH2_RG		port0027
#define	TE54X_USART_A_RAL1_RG		port0028
#define	TE54X_USART_A_RHCR_RG		port0029	// r-only
#define	TE54X_USART_A_RAL2_RG		port0029	// w-only
#define	TE54X_USART_A_RBCL_RG		port002A	// r-only
#define	TE54X_USART_A_XBCL_RG		port002A	// w-only
#define	TE54X_USART_A_RBCH_RG		port002B	// r-only
#define	TE54X_USART_A_XBCH_RG		port002B	// w-only
#define	TE54X_USART_A_CCR0_RG		port002C
#define	TE54X_USART_A_CCR1_RG		port002D
#define	TE54X_USART_A_CCR2_RG		port002E
#define	TE54X_USART_A_CCR3_RG		port002F
#define	TE54X_USART_A_TSAX_RG		port0030
#define	TE54X_USART_A_TSAR_RG		port0031
#define	TE54X_USART_A_XCCR_RG		port0032
#define	TE54X_USART_A_RCCR_RG		port0033
#define	TE54X_USART_A_VSTR_RG		port0034	// r-only
#define	TE54X_USART_A_BGR_RG		port0034	// w-only
#define	TE54X_USART_A_RLCR_RG		port0035
#define	TE54X_USART_A_AML_RG		port0036
#define	TE54X_USART_A_AMH_RG		port0037
#define	TE54X_USART_A_GIS_RG		port0038
#define	TE54X_USART_A_IVA_RG		port0038
#define	TE54X_USART_A_IPC_RG		port0039
#define	TE54X_USART_A_ISR0_RG		port003A	// r-only
#define	TE54X_USART_A_IMR0_RG		port003A	// w-only
#define	TE54X_USART_A_ISR1_RG		port003B	// r-only	
#define	TE54X_USART_A_IMR1_RG		port003B	// w-only
#define	TE54X_USART_A_PVR_RG		port003C
#define	TE54X_USART_A_PIS_RG		port003D	// r-only
#define	TE54X_USART_A_PIM_RG		port003D	// w-only
#define	TE54X_USART_A_PCR_RG		port003E
#define	TE54X_USART_A_CCR4_RG		port003F

			// ASYNC-mode register aliases
#define	TE54X_USART_A_XON_RG		port0024	
#define	TE54X_USART_A_XOFF_RG		port0025
#define	TE54X_USART_A_TCR_RG		port0026
#define	TE54X_USART_A_DAFO_RG		port0027
#define	TE54X_USART_A_RFC_RG		port0028
#define	TE54X_USART_A_TIC_RG		port0035
#define	TE54X_USART_A_MXN_RG		port0036
#define	TE54X_USART_A_MXF_RG		port0037

			// BISYNC-mode register aliases
#define	TE54X_USART_A_SYNL_RG		port0024	
#define	TE54X_USART_A_SYNH_RG		port0025


		// USART channel #B
			// registers for HDLC mode
#define	TE54X_USART_B_RFIFO_RG		port0040	// r-only	
#define	TE54X_USART_B_TFIFO_RG		port0040	// w-only
#define	TE54X_USART_B_STAR_RG		port0060	// r-only
#define	TE54X_USART_B_CMDR_RG		port0060	// w-only
#define	TE54X_USART_B_RSTA_RG		port0061	// r-only
#define	TE54X_USART_B_PRE_RG		port0061	// w-only
#define	TE54X_USART_B_MODE_RG		port0062
#define	TE54X_USART_B_TIMR_RG		port0063
#define	TE54X_USART_B_XAD1_RG		port0064
#define	TE54X_USART_B_XAD2_RG		port0065
#define	TE54X_USART_B_RAH1_RG		port0066
#define	TE54X_USART_B_RAH2_RG		port0067
#define	TE54X_USART_B_RAL1_RG		port0068
#define	TE54X_USART_B_RHCR_RG		port0069	// r-only
#define	TE54X_USART_B_RAL2_RG		port0069	// w-only
#define	TE54X_USART_B_RBCL_RG		port006A	// r-only
#define	TE54X_USART_B_XBCL_RG		port006A	// w-only
#define	TE54X_USART_B_RBCH_RG		port006B	// r-only
#define	TE54X_USART_B_XBCH_RG		port006B	// w-only
#define	TE54X_USART_B_CCR0_RG		port006C
#define	TE54X_USART_B_CCR1_RG		port006D
#define	TE54X_USART_B_CCR2_RG		port006E
#define	TE54X_USART_B_CCR3_RG		port006F
#define	TE54X_USART_B_TSAX_RG		port0070
#define	TE54X_USART_B_TSAR_RG		port0071
#define	TE54X_USART_B_XCCR_RG		port0072
#define	TE54X_USART_B_RCCR_RG		port0073
#define	TE54X_USART_B_VSTR_RG		port0074	// r-only
#define	TE54X_USART_B_BGR_RG		port0074	// w-only
#define	TE54X_USART_B_RLCR_RG		port0075
#define	TE54X_USART_B_AML_RG		port0076
#define	TE54X_USART_B_AMH_RG		port0077
#define	TE54X_USART_B_GIS_RG		port0078
#define	TE54X_USART_B_IVA_RG		port0078
#define	TE54X_USART_B_IPC_RG		port0079
#define	TE54X_USART_B_ISR0_RG		port007A	// r-only
#define	TE54X_USART_B_IMR0_RG		port007A	// w-only
#define	TE54X_USART_B_ISR1_RG		port007B	// r-only	
#define	TE54X_USART_B_IMR1_RG		port007B	// w-only
#define	TE54X_USART_B_PVR_RG		port007C
#define	TE54X_USART_B_PIS_RG		port007D	// r-only
#define	TE54X_USART_B_PIM_RG		port007D	// w-only
#define	TE54X_USART_B_PCR_RG		port007E
#define	TE54X_USART_B_CCR4_RG		port007F

			// ASYNC-mode register aliases
#define	TE54X_USART_B_XON_RG		port0064	
#define	TE54X_USART_B_XOFF_RG		port0065
#define	TE54X_USART_B_TCR_RG		port0066
#define	TE54X_USART_B_DAFO_RG		port0067
#define	TE54X_USART_B_RFC_RG		port0068
#define	TE54X_USART_B_TIC_RG		port0075
#define	TE54X_USART_B_MXN_RG		port0076
#define	TE54X_USART_B_MXF_RG		port0077

			// BISYNC-mode register aliases
#define	TE54X_USART_B_SYNL_RG		port0064	
#define	TE54X_USART_B_SYNH_RG		port0065


	//---- 8-bit SIOX-A rev.C area 
	//     (require XF=1 during access)
	// SIOX-A rev.C is parallel 8-bit interface within the
	// 0x6000..0x603F I/O port address range. Since this
	// area is mapped into I/O space of C54x DSP, and TI
	// C-compiler tools does not support access to this area using
	// function index addressing (only port6000, etc is allowed),
	// then it is impossible to provide universal declarations
	// for this area. User must provide application specific 
	// declarations for this area. Use access macros below.

#define TE54X_SXA_REVC_DATA_BITMASK	0xff

#define	TE54X_SXA_REVC_SADDR		port6000
#define	TE54X_SXA_REVC_EADDR		port603F



	//---- 16-bit PIOX-16 area 
	//     (require XF=0 during access)
	// PIOX-16 is parallel 16-bit interface within the
	// 0x0000..0xffff I/O port address range, which must be
	// addressed with XF=0. 
	// Since this area is mapped into I/O space of C54x DSP, and TI
	// C-compiler tools does not support access to this area using
	// function index addressing (only port0000, etc is allowed),
	// then it is impossible to provide universal declarations
	// for this area. User must provide application specific 
	// declarations for this area. Use access macros below.

#define TE54X_PX_DATA_BITMASK		0xffff

#define	TE54X_PX_SADDR			port0000
#define	TE54X_PX_EADDR			portffff


//!!!!! C54x I/O ports declaration (required for TI C54x C compiler)
//	These defs repeats I/O ports defined above.
ioport	unsigned	port0000;
ioport	unsigned	portffff;
ioport	unsigned	port6000;
ioport	unsigned	port603F;
ioport	unsigned	port8000;
ioport	unsigned	porta000;	
ioport	unsigned	porta001;	
ioport	unsigned	porta002;	
ioport	unsigned	porta003;	
ioport	unsigned	porta004;	
ioport	unsigned	portfffd;
ioport	unsigned	portfffe;
ioport	unsigned	portffff;
ioport	unsigned	port8007;	
ioport	unsigned	port4000;
ioport	unsigned	port8006;
ioport	unsigned	port8001;
ioport	unsigned	port8002;
ioport	unsigned	port8003;
ioport	unsigned	port2000;
ioport	unsigned	port2001;
ioport	unsigned	port2002;
ioport	unsigned	port2003;
ioport	unsigned	port2004;
ioport	unsigned	port2005;
ioport	unsigned	port2006;
ioport	unsigned	port2007;
ioport	unsigned	port2008;
ioport	unsigned	port2009;
ioport	unsigned	port200a;
ioport	unsigned	port200b;
ioport	unsigned	port200c;
ioport	unsigned	port200d;
ioport	unsigned	port200e;
ioport	unsigned	port200f;
ioport	unsigned	port2010;
ioport	unsigned	port2011;
ioport	unsigned	port2012;
ioport	unsigned	port2014;
ioport	unsigned	port2015;
ioport	unsigned	port2016;
ioport	unsigned	port2017;
ioport	unsigned	port2018;
ioport	unsigned	port2019;
ioport	unsigned	port201A;
ioport	unsigned	port201B;
ioport	unsigned	port201C;
ioport	unsigned	port0020;
ioport	unsigned	port0021;
ioport	unsigned	port0022;
ioport	unsigned	port0023;
ioport	unsigned	port0024;
ioport	unsigned	port0025;
ioport	unsigned	port0026;
ioport	unsigned	port0027;
ioport	unsigned	port0028;
ioport	unsigned	port0029;
ioport	unsigned	port002A;
ioport	unsigned	port002B;
ioport	unsigned	port002C;
ioport	unsigned	port002D;
ioport	unsigned	port002E;
ioport	unsigned	port002F;
ioport	unsigned	port0030;
ioport	unsigned	port0031;
ioport	unsigned	port0032;
ioport	unsigned	port0033;
ioport	unsigned	port0034;
ioport	unsigned	port0035;
ioport	unsigned	port0036;
ioport	unsigned	port0037;
ioport	unsigned	port0038;
ioport	unsigned	port0039;
ioport	unsigned	port003A;
ioport	unsigned	port003B;
ioport	unsigned	port003C;
ioport	unsigned	port003D;
ioport	unsigned	port003E;
ioport	unsigned	port003F;
ioport	unsigned	port0040;
ioport	unsigned	port0060;
ioport	unsigned	port0061;
ioport	unsigned	port0062;
ioport	unsigned	port0063;
ioport	unsigned	port0064;
ioport	unsigned	port0065;
ioport	unsigned	port0066;
ioport	unsigned	port0067;
ioport	unsigned	port0068;
ioport	unsigned	port0069;
ioport	unsigned	port006A;
ioport	unsigned	port006B;
ioport	unsigned	port006C;
ioport	unsigned	port006D;
ioport	unsigned	port006E;
ioport	unsigned	port006F;
ioport	unsigned	port0070;
ioport	unsigned	port0071;
ioport	unsigned	port0072;
ioport	unsigned	port0073;
ioport	unsigned	port0074;
ioport	unsigned	port0075;
ioport	unsigned	port0076;
ioport	unsigned	port0077;
ioport	unsigned	port0078;
ioport	unsigned	port0079;
ioport	unsigned	port007A;
ioport	unsigned	port007B;
ioport	unsigned	port007C;
ioport	unsigned	port007D;
ioport	unsigned	port007E;
ioport	unsigned	port007F;
//=====================================================================





//=================== I/O ports constants =============================

	//---- XDMP rg defs 
#define	TE54X_XDM_PAGE_BITMASK		0x1f
#define	TE54X_XDM_SEL_BITMASK		0x80

#define	TE54X_XDM_SEL_DRAM		0x00
#define	TE54X_XDM_SEL_FLASH		0x80


	//----- MIRQ-0..3/NMI selector codes
	//	(unused codes are reserved and will result in no IRQ source)
#define	TE54X_MIRQ_SEL_BITMASK		0xf	// read-back bitmask

#define	TE54X_MIRQ_SEL_OFF		0x0	// no IRQ source, power-on default
#define	TE54X_MIRQ_SEL_USART		0x1	
#define	TE54X_MIRQ_SEL_USB		0x2
#define	TE54X_MIRQ_SEL_WDT		0x3	
#define	TE54X_MIRQ_SEL_SXIRQ0		0x4
#define	TE54X_MIRQ_SEL_SXIRQ1		0x5
#define	TE54X_MIRQ_SEL_SXIRQ2		0x6
#define	TE54X_MIRQ_SEL_PXIRQ0		0x7
#define	TE54X_MIRQ_SEL_PXIRQ1		0x8
	

	//---- DEV_ID_RG read-only register
#define	TE54X_DEV_ID_BITMASK		0x07
#define	TE54X_REV_ID_BITMASK		0x70
#define	TE54X_DEV_ID_BITSHIFT		0
#define	TE54X_REV_ID_BITSHIFT		4

		// DEVICE ID codes
#define	TE548_DEV_ID			0x01	
#define	TE549_DEV_ID			0x02	
#define	TE5402_DEV_ID			0x03	
#define	TE5409_DEV_ID			0x04	
#define	TE5410_DEV_ID			0x05	
#define	TE5416_DEV_ID			0x06	

		// REV ID codes
#define	TE54X_REV_ID_REV1		0x10	
#define	TE54X_LAST_REV_ID		0x10	// most recent REV ID code	


	//---- SYS_STAT_RG read-only register
#define	TE54X_BMODE_BITMASK		0x3
#define	TE54X_HPI_ENABLE_BITMASK	0x8

		// start-up bootmode (latched on release of DSP reset)
#define	TE54X_BMODE_NONE		0x0	// no boot, MP-mode
#define	TE54X_BMODE_FLASH8		0x1	// boot from 8-bit FLASH/EPROM, MC-mode
#define	TE54X_BMODE_HPI			0x2	// bot from HPI, MC-mode
#define	TE54X_BMODE_NONE1		0x3	// reserved: no boot, MP-mode

		// HPI enable (latched on release of DSP reset)
		// (might be used to recognize HPI disable in order to
		//  enable MTIMER-1 via TM/XIO-1 for E5402 and to use
		//  8-bit HPI data bus as GPIO for E5402/E5416)
#define	TE54X_HPI_ENABLE_ON		0x8	// HPI is enabled
#define	TE54X_HPI_ENABLE_OFF		0x0	// HPI is disabled
#define	TE54X_HPI_ENABLE_BITSHIFT	0x3	// bitshifts for universal macros (ON/OFF)


	//---- WDT enable register
#define	TE54X_WDT_ENABLE_BITMASK	0x1
#define	TE54X_WDT_ENABLE_OFF		0x0
#define	TE54X_WDT_ENABLE_ON		0x1
#define	TE54X_WDT_ENABLE_BITSHIFT	0	// bitshifts for universal macros (ON/OFF)


	//--- PIOX/SIOX Reset Control via PXSX_RESET_RG register
#define	TE54X_PX_RESET_BITMASK		1	// PIOX-16 reset control
#define	TE54X_SXA_RESET_BITMASK		2	// SIOX-A reset control
#define	TE54X_SXB_RESET_BITMASK		4	// SIOX-B reset control

#define	TE54X_PX_RESET_ON		0
#define	TE54X_PX_RESET_OFF		1

#define	TE54X_SXA_RESET_ON		0
#define	TE54X_SXA_RESET_OFF		2

#define	TE54X_SXB_RESET_ON		0
#define	TE54X_SXB_RESET_OFF		4

#define	TE54X_PX_RESET_BITSHIFT		0	// bitshifts for universal macros
#define	TE54X_SXA_RESET_BITSHIFT	1
#define	TE54X_SXB_RESET_BITSHIFT	2
#define	TE54X_PXSX_RESET_ON		0	// universal defs if used via shift bits
#define	TE54X_PXSX_RESET_OFF		1



	//---- PIOX/SIOX TM/XIO-0/1 pins control
	//     via XIO_CNF_RG, XIO_DIR_RG and XIO_DATA_RG
		// TM/XIO pins configuration via XIO_CNF_RG
#define	TE54X_XIO0_CNF_BITMASK		1
#define	TE54X_XIO1_CNF_BITMASK		2

#define	TE54X_XIO0_CNF_TIMER		1
#define	TE54X_XIO0_CNF_IO		0

#define	TE54X_XIO1_CNF_TIMER		2	// E5402 only with HPI disabled
#define	TE54X_XIO1_CNF_IO		0

#define	TE54X_XIO0_CNF_BITSHIFT		0	// bitshifts for universal macros
#define	TE54X_XIO1_CNF_BITSHIFT		1
#define	TE54X_XIO_CNF_TIMER		1	// universal defs for bitshifts
#define	TE54X_XIO_CNF_IO		0


		// TM/XIO pins direction control via XIO_DIR_RG if configured as I/O
#define	TE54X_XIO0_DIR_BITMASK		1
#define	TE54X_XIO1_DIR_BITMASK		2

#define	TE54X_XIO0_DIR_OUTPUT		1
#define	TE54X_XIO0_DIR_INPUT		0

#define	TE54X_XIO1_DIR_OUTPUT		2
#define	TE54X_XIO1_DIR_INPUT		0

#define	TE54X_XIO0_DIR_BITSHIFT		0	// bitshifts for universal macros
#define	TE54X_XIO1_DIR_BITSHIFT		1
#define	TE54X_XIO_DIR_OUTPUT		1	// universal defs for bitshifts
#define	TE54X_XIO_DIR_INPUT		0


		// TM/XIO pins data control via XIO_DATA_RG if configured as I/O
#define	TE54X_XIO0_DATA_BITMASK		1
#define	TE54X_XIO1_DATA_BITMASK		2

#define	TE54X_XIO0_DATA_ON		1
#define	TE54X_XIO0_DATA_OFF		0

#define	TE54X_XIO1_DATA_ON		2
#define	TE54X_XIO1_DATA_OFF		0

#define	TE54X_XIO0_DATA_BITSHIFT	0	// bitshifts for universal macros (ON/OFF)
#define	TE54X_XIO1_DATA_BITSHIFT	1
#define	TE54X_XIO_DATA_ON		1	// universal defs for bitshifts
#define	TE54X_XIO_DATA_OFF		0

//=====================================================================


//====================== Functions and Macros =========================
// Utility functions and macros provided below shall be used for access
// to DSP I/O ports and memory of TORNADO-E54x.
// Utility functions comprise of several function groups:
//	- DSP init
//	- utility functions for accessing I/O ports
//	- AMD FLASH memory programming utilities
// User can control inclusion of function bodies for desired function 
// groups during compilation procedure by means of setting glabal defs.
// This allows either to reduce code size by exclusion of undesired code,
// or to exclude multiple function bodies in case user project comprises
// of several .OBJ files, which are using common utilities.



//---- DSP XF set/reset macros, which shall be used for access to TE54x I/O regs/areas
#define __C54X_SET_XF__		asm(" ssbx xf")		// sets XF=1, default setting after DSP reset
#define __C54X_RESET_XF__	asm(" rsbx xf")		// sets XF=0.

#define	TE54X_SET_NOTPX_IO_AREA	__C54X_SET_XF__		// This macros sets XF=1 and must be included into 
							// user application code prior accessing all 
							// I/O regs/areas except for PIOX-16. 

#define	TE54X_SET_PX_IO_AREA	__C54X_RESET_XF__	// This macros sets XF=0 and must be included into 
							// user application code prior accessing PIOX-16
							// I/O area.


//---- control registers/areas access macros ----

	//>>> XDMP register control (requires TE54X_SET_NOTPX_IO_AREA)
#define TE54X_set_xdmp_rg(v)		TE54X_XDMP_RG=v	// V={TE54X_XDM_SEL_{DRAM/FLASH} | xdm_page# }
#define TE54X_get_xdmp_rg()		(TE54X_XDMP_RG & TE54X_CNTR_DATA_BITMASK)
		// dedicated field/bit-specific macros
			// v= xdm_page#
#define TE54X_set_xdm_page(v)		TE54X_set_xdmp_rg((v & TE54X_XDM_PAGE_BITMASK) | (TE54X_get_xdmp_rg() & ~TE54X_XDM_PAGE_BITMASK))
#define TE54X_get_xdm_page()		(TE54X_get_xdmp_rg() & TE54X_XDM_PAGE_BITMASK)
			// v={TE54X_XDM_SEL_{DRAM/FLASH}}
#define TE54X_set_xdm_sel(v)		TE54X_set_xdmp_rg((v & TE54X_XDM_SEL_BITMASK) | (TE54X_get_xdmp_rg() & ~TE54X_XDM_SEL_BITMASK))
#define TE54X_get_xdm_sel()		(TE54X_get_xdmp_rg() & TE54X_XDM_SEL_BITMASK)
			// direct XDM selection/status macros
#define TE54X_SET_XDM_SEL_DRAM		TE54X_set_xdm_sel(TE54X_XDM_SEL_DRAM)
#define TE54X_SET_XDM_SEL_FLASH		TE54X_set_xdm_sel(TE54X_XDM_SEL_FLASH)
#define TE54X_XDM_SEL_IS_DRAM		(TE54X_get_xdm_sel()==TE54X_XDM_SEL_DRAM)
#define TE54X_XDM_SEL_IS_FLASH		(TE54X_get_xdm_sel()==TE54X_XDM_SEL_FLASH)



	//>>> WDT control (requires TE54X_SET_NOTPX_IO_AREA)
#define	TE54X_set_wdt_reset()		TE54X_WDT_RESET_RG=0

#define	TE54X_set_wdt_enable_rg(v)	TE54X_WDT_ENABLE_RG=v	// V={TE54X_ON,TE54X_OFF}
#define	TE54X_get_wdt_enable_rg()	(TE54X_WDT_ENABLE_RG & TE54X_CNTR_DATA_BITMASK)
		// dedicated bit-specific macros with V = {TE54X_WDT_ENABLE_ON,TE54X_WDT_ENABLE_OFF}
#define	TE54X_set_wdt_enable(v)		TE54X_set_wdt_enable_rg(v)
#define	TE54X_get_wdt_enable()		TE54X_get_wdt_enable_rg()
		// Unified macros with V = {TE54X_ON,TE54X_OFF} (same as bit-specific macros)
#define	TE54X_uset_wdt_enable(v)	TE54X_set_wdt_enable_rg(v)
#define	TE54X_uget_wdt_enable()		TE54X_get_wdt_enable_rg()
		// direct set/get macros
#define	TE54X_SET_WDT_ENABLE_OFF	TE54X_set_wdt_enable_rg(TE54X_WDT_ENABLE_OFF)
#define	TE54X_SET_WDT_ENABLE_ON		TE54X_set_wdt_enable_rg(TE54X_WDT_ENABLE_ON)
#define	TE54X_WDT_ENABLE_IS_OFF		(TE54X_get_wdt_enable_rg()==TE54X_WDT_ENABLE_OFF)
#define	TE54X_WDT_ENABLE_IS_ON		(TE54X_get_wdt_enable_rg()==TE54X_WDT_ENABLE_ON)


	//>>> PIOX/SIOX reset control (requires TE54X_SET_NOTPX_IO_AREA)
#define	TE54X_set_pxsx_reset_rg(v)	TE54X_PXSX_RESET_RG=v	// V={TE54X_PX_RESET_ON/OFF | TE54X_SXA_RESET_ON/OFF | TE54X_SXB_RESET_ON/OFF }
#define	TE54X_get_pxsx_reset_rg()	(TE54X_PXSX_RESET_RG & TE54X_CNTR_DATA_BITMASK)
		// dedicated bit-specific macros with V = {TE54X_{PX/SXA/SXB}_RESET_ON,TE54X_{PX/SXA/SXB}_RESET_OFF}
#define	TE54X_set_px_reset(v)		TE54X_set_pxsx_reset_rg((TE54X_get_pxsx_reset_rg() & ~TE54X_PX_RESET_BITMASK) | (v & TE54X_PX_RESET_BITMASK))	
#define	TE54X_get_px_reset()		(TE54X_get_pxsx_reset_rg() & TE54X_PX_RESET_BITMASK)
#define	TE54X_set_sxa_reset(v)		TE54X_set_pxsx_reset_rg((TE54X_get_pxsx_reset_rg() & ~TE54X_SXA_RESET_BITMASK) | (v & TE54X_SXA_RESET_BITMASK))	
#define	TE54X_get_sxa_reset()		(TE54X_get_pxsx_reset_rg() & TE54X_SXA_RESET_BITMASK)
#define	TE54X_set_sxb_reset(v)		TE54X_set_pxsx_reset_rg((TE54X_get_pxsx_reset_rg() & ~TE54X_SXB_RESET_BITMASK) | (v & TE54X_SXB_RESET_BITMASK))	
#define	TE54X_get_sxb_reset()		(TE54X_get_pxsx_reset_rg() & TE54X_SXB_RESET_BITMASK)
		// Unified macros with V = {TE54X_PXSX_RESET_ON,TE54X_PXSX_RESET_OFF}
#define	TE54X_uset_px_reset(v)		TE54X_set_px_reset(v)
#define	TE54X_uget_px_reset()		TE54X_get_px_reset()
#define	TE54X_uset_sxa_reset(v)		TE54X_set_sxa_reset((v<<TE54X_SXA_RESET_BITSHIFT))	
#define	TE54X_uget_sxa_reset()		(TE54X_get_sxa_reset()>>TE54X_SXA_RESET_BITSHIFT)
#define	TE54X_uset_sxb_reset(v)		TE54X_set_sxb_reset((v<<TE54X_SXB_RESET_BITSHIFT))	
#define	TE54X_uget_sxb_reset()		(TE54X_get_sxb_reset()>>TE54X_SXB_RESET_BITSHIFT)


	//>>> DEV_ID register (requires TE54X_SET_NOTPX_IO_AREA)
#define	TE54X_get_dev_id_rg()		(TE54X_DEV_ID_RG & TE54X_CNTR_DATA_BITMASK)
#define	TE54X_get_dev_id()		(TE54X_DEV_ID_RG & TE54X_DEV_ID_BITMASK)
#define	TE54X_get_rev_id()		(TE54X_DEV_ID_RG & TE54X_REV_ID_BITMASK)

	//>>> SYS_STAT register (requires TE54X_SET_NOTPX_IO_AREA)
#define	TE54X_get_sys_stat_rg()		(TE54X_SYS_STAT_RG & TE54X_CNTR_DATA_BITMASK)
#define	TE54X_get_bmode()		(TE54X_SYS_STAT_RG & TE54X_BMODE_BITMASK)
#define	TE54X_get_hpi_enable()		(TE54X_SYS_STAT_RG & TE54X_HPI_ENABLE_BITMASK)
		// Unified macros with V={TE54X_ON,TE54X_OFF} 
#define	TE54X_uget_hpi_enable()		(TE54X_get_hpi_enable()>>TE54X_HPI_ENABLE_BITSHIFT)
		// board ID macros
#define	TE548_BOARD			(TE54X_get_dev_id()==TE548_DEV_ID)
#define	TE549_BOARD			(TE54X_get_dev_id()==TE549_DEV_ID)
#define	TE5402_BOARD			(TE54X_get_dev_id()==TE5402_DEV_ID)
#define	TE5409_BOARD			(TE54X_get_dev_id()==TE5409_DEV_ID)
#define	TE5410_BOARD			(TE54X_get_dev_id()==TE5410_DEV_ID)
#define	TE5416_BOARD			(TE54X_get_dev_id()==TE5416_DEV_ID)
		// Boot mode ID macros
#define	TE54X_BMODE_IS_NONE		((TE54X_get_bmode()==TE54X_BMODE_NONE) | (TE54X_get_bmode()==TE54X_BMODE_NONE1))
#define	TE54X_BMODE_IS_FLASH8		(TE54X_get_bmode()==TE54X_BMODE_FLASH8)
#define	TE54X_BMODE_IS_HPI		(TE54X_get_bmode()==TE54X_BMODE_HPI)
		// HPI enable ID macros
#define	TE54X_HPI_ENABLE_IS_OFF		(TE54X_get_hpi_enable()==TE54X_HPI_ENABLE_OFF)
#define	TE54X_HPI_ENABLE_IS_ON		(TE54X_get_hpi_enable()==TE54X_HPI_ENABLE_ON)


	//>>> MIRQ-0..3 and NMI source selectors (require TE54X_SET_NOTPX_IO_AREA)
#define	TE54X_set_mirq0_sel_rg(v)	TE54X_MIRQ0_SEL_RG=v	//V={TE54X_MIRQ_SEL_..}
#define	TE54X_get_mirq0_sel_rg()	(TE54X_MIRQ0_SEL_RG & TE54X_CNTR_DATA_BITMASK)
				// macros aliases
#define	TE54X_set_mirq0_sel(v)		TE54X_set_mirq0_sel_rg(v)
#define	TE54X_get_mirq0_sel()		TE54X_get_mirq0_sel_rg()

#define	TE54X_set_mirq1_sel_rg(v)	TE54X_MIRQ1_SEL_RG=v	//V={TE54X_MIRQ_SEL_..}
#define	TE54X_get_mirq1_sel_rg()	(TE54X_MIRQ1_SEL_RG & TE54X_CNTR_DATA_BITMASK)
				// macros aliases
#define	TE54X_set_mirq1_sel(v)		TE54X_set_mirq1_sel_rg(v)
#define	TE54X_get_mirq1_sel()		TE54X_get_mirq1_sel_rg()

#define	TE54X_set_mirq2_sel_rg(v)	TE54X_MIRQ2_SEL_RG=v	//V={TE54X_MIRQ_SEL_..}
#define	TE54X_get_mirq2_sel_rg()	(TE54X_MIRQ2_SEL_RG & TE54X_CNTR_DATA_BITMASK)
				// macros aliases
#define	TE54X_set_mirq2_sel(v)		TE54X_set_mirq2_sel_rg(v)
#define	TE54X_get_mirq2_sel()		TE54X_get_mirq2_sel_rg()

#define	TE54X_set_mirq3_sel_rg(v)	TE54X_MIRQ3_SEL_RG=v	//V={TE54X_MIRQ_SEL_..}
#define	TE54X_get_mirq3_sel_rg()	(TE54X_MIRQ3_SEL_RG & TE54X_CNTR_DATA_BITMASK)
				// macros aliases
#define	TE54X_set_mirq3_sel(v)		TE54X_set_mirq3_sel_rg(v)
#define	TE54X_get_mirq3_sel()		TE54X_get_mirq3_sel_rg()

#define	TE54X_set_mnmi_sel_rg(v)	TE54X_MNMI_SEL_RG=v	//V={TE54X_MIRQ_SEL_..}
#define	TE54X_get_mnmi_sel_rg()		(TE54X_MNMI_SEL_RG & TE54X_CNTR_DATA_BITMASK)
				// macros aliases
#define	TE54X_set_mnmi_sel(v)		TE54X_set_mnmi_sel_rg(v)
#define	TE54X_get_mnmi_sel()		TE54X_get_mnmi_sel_rg()

		// universal macros to set/get MIRQ/MNMI selector regs using MIRQ ID
		// IDs for MIRQ/NMI selector registers
#define	TE54X_MIRQ0_SEL			0
#define	TE54X_MIRQ1_SEL			1
#define	TE54X_MIRQ2_SEL			2
#define	TE54X_MIRQ3_SEL			3
#define	TE54X_MNMI_SEL			4

			//input params: IRQ={TE54X_MIRQx_SEL/TE54X_MNMI_SEL}, V={TE54X_MIRQ_SEL_..}
#define	TE54X_set_mirq_sel_rg(irq,v)	if(irq==0) TE54X_set_mirq0_sel(v); \
					  else if(irq==1) TE54X_set_mirq1_sel(v);\
					    else if(irq==2) TE54X_set_mirq2_sel(v);\
					      else if(irq==3) TE54X_set_mirq3_sel(v);\
						else TE54X_set_mnmi_sel(v)
#define	TE54X_get_mirq_sel_rg(irq)	((irq==0)? TE54X_get_mirq0_sel(): \
					  ((irq==1)? TE54X_get_mirq1_sel():\
					    ((irq==2)? TE54X_get_mirq2_sel():\
					      ((irq==3)? TE54X_get_mirq3_sel():TE54X_get_mnmi_sel()))))
				// macros aliases
#define	TE54X_set_mirq_sel(irq,v)	TE54X_set_mirq_sel_rg(irq,v)
#define	TE54X_get_mirq_sel(irq)		TE54X_get_mirq_sel_rg(irq)



	//>>> TM/XIO-0/1 pins control (requires TE54X_SET_NOTPX_IO_AREA)
		// XIO pins configuration control
#define	TE54X_set_xio_cnf_rg(v)		TE54X_XIO_CNF_RG=v	// V={TE54X_XIO0_CNF_{TIMER/IO} | TE54X_XIO1_CNF_{TIMER/IO}}
#define	TE54X_get_xio_cnf_rg()		(TE54X_XIO_CNF_RG & TE54X_CNTR_DATA_BITMASK)
			// dedicated bit-specific macros with V={TE54X_XIO{0/1}_CNF_{TIMER/IO}}
#define	TE54X_set_xio0_cnf(v)		TE54X_set_xio_cnf_rg((TE54X_get_xio_cnf_rg() & ~TE54X_XIO0_CNF_BITMASK) | (v & TE54X_XIO0_CNF_BITMASK))
#define	TE54X_get_xio0_cnf()		(TE54X_get_xio_cnf_rg() & TE54X_XIO0_CNF_BITMASK)
#define	TE54X_set_xio1_cnf(v)		TE54X_set_xio_cnf_rg((TE54X_get_xio_cnf_rg() & ~TE54X_XIO1_CNF_BITMASK) | (v & TE54X_XIO1_CNF_BITMASK))
#define	TE54X_get_xio1_cnf()		(TE54X_get_xio_cnf_rg() & TE54X_XIO1_CNF_BITMASK)
		// Unified macros with V = {TE54X_XIO_CNF_{TIMER/IO}}
#define	TE54X_uset_xio0_cnf(v)		TE54X_set_xio0_cnf(v)
#define	TE54X_uget_xio0_cnf()		TE54X_get_xio0_cnf()
#define	TE54X_uset_xio1_cnf(v)		TE54X_set_xio1_cnf((v<<TE54X_XIO1_CNF_BITSHIFT))
#define	TE54X_uget_xio1_cnf()		(TE54X_get_xio1_cnf()>>TE54X_XIO1_CNF_BITSHIFT)
		// dedicated macros
#define	TE54X_SET_XIO0_CNF_IO		TE54X_uset_xio0_cnf(TE54X_XIO_CNF_IO)
#define	TE54X_SET_XIO0_CNF_TIMER	TE54X_uset_xio0_cnf(TE54X_XIO_CNF_TIMER)
#define	TE54X_SET_XIO1_CNF_IO		TE54X_uset_xio1_cnf(TE54X_XIO_CNF_IO)
#define	TE54X_SET_XIO1_CNF_TIMER	TE54X_uset_xio1_cnf(TE54X_XIO_CNF_TIMER)
#define	TE54X_XIO0_CNF_IS_IO		(TE54X_uget_xio0_cnf()==TE54X_XIO_CNF_IO)
#define	TE54X_XIO0_CNF_IS_TIMER		(TE54X_uget_xio0_cnf()==TE54X_XIO_CNF_TIMER)
#define	TE54X_XIO1_CNF_IS_IO		(TE54X_uget_xio1_cnf()==TE54X_XIO_CNF_IO)
#define	TE54X_XIO1_CNF_IS_TIMER		(TE54X_uget_xio1_cnf()==TE54X_XIO_CNF_TIMER)


		// XIO pins direction control (if configured as I/O)
#define	TE54X_set_xio_dir_rg(v)		TE54X_XIO_DIR_RG=v	// V={TE54X_XIO0_DIR_{INPUT/OUTPUT} | TE54X_XIO1_DIR_{INPUT/OUTPUT}}
#define	TE54X_get_xio_dir_rg()		(TE54X_XIO_DIR_RG & TE54X_CNTR_DATA_BITMASK)
			// dedicated bit-specific macros with V={TE54X_XIO{0/1}_DIR_{INPUT/OUTPUT}
#define	TE54X_set_xio0_dir(v)		TE54X_set_xio_dir_rg((TE54X_get_xio_dir_rg() & ~TE54X_XIO0_DIR_BITMASK) | (v & TE54X_XIO0_DIR_BITMASK))
#define	TE54X_get_xio0_dir()		(TE54X_get_xio_dir_rg() & TE54X_XIO0_DIR_BITMASK)
#define	TE54X_set_xio1_dir(v)		TE54X_set_xio_dir_rg((TE54X_get_xio_dir_rg() & ~TE54X_XIO1_DIR_BITMASK) | (v & TE54X_XIO1_DIR_BITMASK))
#define	TE54X_get_xio1_dir()		(TE54X_get_xio_dir_rg() & TE54X_XIO1_DIR_BITMASK)
		// Unified macros with V = {TE54X_XIO_DIR_{INPUT/OUTPUT}}
#define	TE54X_uset_xio0_dir(v)		TE54X_set_xio0_dir(v)
#define	TE54X_uget_xio0_dir()		TE54X_get_xio0_dir()
#define	TE54X_uset_xio1_dir(v)		TE54X_set_xio1_dir((v<<TE54X_XIO1_DIR_BITSHIFT))
#define	TE54X_uget_xio1_dir()		(TE54X_get_xio1_dir()>>TE54X_XIO1_DIR_BITSHIFT)
		// dedicated macros
#define	TE54X_SET_XIO0_DIR_INPUT	TE54X_uset_xio0_dir(TE54X_XIO_DIR_INPUT)
#define	TE54X_SET_XIO0_DIR_OUTPUT	TE54X_uset_xio0_dir(TE54X_XIO_DIR_OUTPUT)
#define	TE54X_SET_XIO1_DIR_INPUT	TE54X_uset_xio1_dir(TE54X_XIO_DIR_INPUT)
#define	TE54X_SET_XIO1_DIR_OUTPUT	TE54X_uset_xio1_dir(TE54X_XIO_DIR_OUTPUT)
#define	TE54X_XIO0_DIR_IS_INPUT		(TE54X_uget_xio0_dir()==TE54X_XIO_DIR_INPUT)
#define	TE54X_XIO0_DIR_IS_OUTPUT	(TE54X_uget_xio0_dir()==TE54X_XIO_DIR_OUTPUT)
#define	TE54X_XIO1_DIR_IS_INPUT		(TE54X_uget_xio1_dir()==TE54X_XIO_DIR_INPUT)
#define	TE54X_XIO1_DIR_IS_OUTPUT	(TE54X_uget_xio1_dir()==TE54X_XIO_DIR_OUTPUT)


		// XIO pins data control (if configured as I/O)
#define	TE54X_set_xio_data_rg(v)	TE54X_XIO_DATA_RG=v	// V={TE54X_XIO0_DATA_{ON/OFF} | TE54X_XIO1_DATA_{ON/OFF}}
#define	TE54X_get_xio_data_rg()		(TE54X_XIO_DATA_RG & TE54X_CNTR_DATA_BITMASK)
			// dedicated bit-specific macros with V={TE54X_XIO{0/1}_DATA_{ON/OFF}
#define	TE54X_set_xio0_data(v)		TE54X_set_xio_data_rg((TE54X_get_xio_data_rg() & ~TE54X_XIO0_DATA_BITMASK) | (v & TE54X_XIO0_DATA_BITMASK))
#define	TE54X_get_xio0_data()		(TE54X_get_xio_data_rg() & TE54X_XIO0_DATA_BITMASK)
#define	TE54X_set_xio1_data(v)		TE54X_set_xio_data_rg((TE54X_get_xio_data_rg() & ~TE54X_XIO1_DATA_BITMASK) | (v & TE54X_XIO1_DATA_BITMASK))
#define	TE54X_get_xio1_data()		(TE54X_get_xio_data_rg() & TE54X_XIO1_DATA_BITMASK)
		// Unified macros with V = {TE54X_XIO_DATA_{ON/OFF}}
#define	TE54X_uset_xio0_data(v)		TE54X_set_xio0_data(v)
#define	TE54X_uget_xio0_data()		TE54X_get_xio0_data()
#define	TE54X_uset_xio1_data(v)		TE54X_set_xio1_data((v<<TE54X_XIO1_DATA_BITSHIFT))	
#define	TE54X_uget_xio1_data()		(TE54X_get_xio1_data()>>TE54X_XIO1_DATA_BITSHIFT)
		// dedicated macros
#define	TE54X_SET_XIO0_DATA_OFF		TE54X_uset_xio0_data(TE54X_XIO_DATA_OFF)
#define	TE54X_SET_XIO0_DATA_ON		TE54X_uset_xio0_data(TE54X_XIO_DATA_ON)
#define	TE54X_SET_XIO1_DATA_OFF		TE54X_uset_xio1_data(TE54X_XIO_DATA_OFF)
#define	TE54X_SET_XIO1_DATA_ON		TE54X_uset_xio1_data(TE54X_XIO_DATA_ON)
#define	TE54X_XIO0_DATA_IS_OFF		(TE54X_uget_xio0_data()==TE54X_XIO_DATA_OFF)
#define	TE54X_XIO0_DATA_IS_ON		(TE54X_uget_xio0_data()==TE54X_XIO_DATA_ON)
#define	TE54X_XIO1_DATA_IS_OFF		(TE54X_uget_xio1_data()==TE54X_XIO_DATA_OFF)
#define	TE54X_XIO1_DATA_IS_ON		(TE54X_uget_xio1_data()==TE54X_XIO_DATA_ON)


	//>>> USART control (requires TE54X_SET_NOTPX_IO_AREA)
	//    rg={TE54X_USART_A/B_xxx_RG}, V={8-bit data}
#define	TE54X_set_usart_rg(rg,v)	rg=v	
#define	TE54X_get_usart_rg(rg)		(rg & TE54X_USART_DATA_BITMASK)
			// macros aliases
#define	TE54X_set_usart(rg,v)		TE54X_set_usart_rg(rg,v)
#define	TE54X_get_usart(rg)		TE54X_get_usart_rg(rg)

	//>>> USB control (requires TE54X_SET_NOTPX_IO_AREA)
	//    rg={TE54X_USB_xxx_RG}, V={8-bit data}
#define	TE54X_set_usb_rg(rg,v)		rg=v	
#define	TE54X_get_usb_rg(rg)		(rg & TE54X_USB_DATA_BITMASK)
			// macros aliases
#define	TE54X_set_usb(rg,v)		TE54X_set_usb_rg(rg,v)
#define	TE54X_get_usb(rg)		TE54X_get_usb_rg(rg)

	//>>> SIOX-A rev.C control (requires TE54X_SET_NOTPX_IO_AREA)
	//     rg={SIOX I/O port}, V={8-bit data}
#define	TE54X_set_sxa_revc_rg(rg,v)	rg=v
#define	TE54X_get_sxa_revc_rg(rg)	(rg & TE54X_SXA_REVC_DATA_BITMASK)
			// macros aliases
#define	TE54X_set_sxa_revc(rg,v)	TE54X_set_sxa_revc_rg(rg,v)
#define	TE54X_get_sxa_revc(rg)		TE54X_get_sxa_revc_rg(rg)
#define	TE54X_set_sx(rg,v)		TE54X_set_sxa_revc_rg(rg,v)
#define	TE54X_get_sx(rg)		TE54X_get_sxa_revc_rg(rg)

	//>>> PIOX-16 control (requires TE54X_SET_PX_IO_AREA)
	//    rg={PIOX-16 I/O port}, V={16-bit data}
#define	TE54X_set_px_rg(rg,v)		rg=v
#define	TE54X_get_px_rg(rg)		(rg & TE54X_SXA_REVC_DATA_BITMASK)
			// macros aliases
#define	TE54X_set_px(rg,v)		TE54X_set_px_rg(rg,v)
#define	TE54X_get_px(rg)		TE54X_get_px_rg(rg)


//---- utility functions
//     Function' code can be included/excluded into user code during compilation process
//     by means of setting run-time compiler defs.

	//>>> E54x and DSP init functions
	//    Inclusion/exclusion of function bodies into user code is controlled via
	//    __TE54X_EXCLUDE_INIT_DSP__ run-time compiler def:
	//	 - if not defined, then function code is included (default)
	//	 - if defined to any value, then function code is excluded
	//    Each init function has XDM_SEL input parameter:
	// 	If XDM_SEL is TE54X_XDM_SEL_DRAM/TE54X_XDM_SEL_FLASH, then
	// 	XDMP_RG is configured for DRAM/FLASH correspondingly and
	// 	C54x DSP on-chip SWWSR register is configured correspondingly.
	// 	If XDM_SEL==TE54_XDM_SEL_AUTO, then D-MEM type is defined via
	// 	XDM_SEL bit of XDMP_RG and C54x DSP on-chip SWWSR register is 
	// 	configured correspondingly.
void	TE54X_init(unsigned xdm_sel);		// performs init of E54x control regs and all DSP on-chip WS control regs
void	TE54X_init_dsp(unsigned xdm_sel);	// init all DSP on-chip WS control regs
void	TE54X_reinit_dsp(unsigned xdm_sel);	// init DSP on-chip SWWSR reg only

		// T54X_XDM_SEL_DRAM and T54X_XDM_SEL_FLASH are defined earlier 
		// in this file for defs of XDM_SEL bit of XDMP_RG
#ifndef	TE54X_XDM_SEL_DRAM			
#define	TE54X_XDM_SEL_DRAM		0x00
#endif
#ifndef	TE54X_XDM_SEL_FLASH
#define	TE54X_XDM_SEL_FLASH		0x80
#endif
#define	TE54X_XDM_SEL_AUTO		0xffff	// extension def: XDM type is read fm the XDM_SEL bit of XDMP_RG

		// direct init function aliases defs
#define	TE54X_init_for_xdm_dram()		TE54X_init(TE54X_XDM_SEL_DRAM)
#define	TE54X_init_for_xdm_flash()		TE54X_init(TE54X_XDM_SEL_FLASH)
#define	TE54X_init_for_xdm_auto()		TE54X_init(TE54X_XDM_SEL_AUTO)

#define	TE54X_init_dsp_for_xdm_dram()		TE54X_init_dsp(TE54X_XDM_SEL_DRAM)
#define	TE54X_init_dsp_for_xdm_flash()		TE54X_init_dsp(TE54X_XDM_SEL_FLASH)
#define	TE54X_init_dsp_for_xdm_auto()		TE54X_init_dsp(TE54X_XDM_SEL_AUTO)

#define	TE54X_reinit_dsp_for_xdm_dram()		TE54X_reinit_dsp(TE54X_XDM_SEL_DRAM)
#define	TE54X_reinit_dsp_for_xdm_flash()	TE54X_reinit_dsp(TE54X_XDM_SEL_FLASH)
#define	TE54X_reinit_dsp_for_xdm_auto()		TE54X_reinit_dsp(TE54X_XDM_SEL_AUTO)


	//>>> FLASH programming utilities
	//    Inclusion/exclusion of function' bodies into user code is controlled via
	//    __TE54X_INCLUDE_AMD_FLASH_CONTROL__ run-time compiler def:
	//	 - if not defined, then function' code is not included (default)
	//	 - if defined to any value, then function' code is included
		// AMD FLASH defs
#define	TE54X_FLASH_AMD_MANUF_ID	0x01
#define	TE54X_FLASH_AM29F010_DEV_ID	0x20
#define TE54X_FLASH_AM29F040_DEV_ID	0xa4
#define TE54X_FLASH_SECTOR_PROTECTED	1
#define TE54X_FLASH_SECTOR_UNPROTECTED	0
#define TE54X_FLASH_DATA_BITMASK	0xff

int 	TE54X_flash_info(unsigned *manuf_id, unsigned *dev_id,unsigned long *len,
			 unsigned long *sectors, unsigned long *sector_len);
int 	TE54X_flash_set_page(unsigned long a,unsigned *fpage, unsigned **fptr);
int 	TE54X_flash_erase_sector(unsigned long addr,unsigned verif_flag);
int 	TE54X_flash_erase_chip(unsigned verif_flag);
int 	TE54X_flash_write(unsigned long addr, unsigned data);
int 	TE54X_flash_read(unsigned long addr);
int 	TE54X_flash_sector_wrprot(unsigned long addr);

		//--- general extended API functions
int 	TE54X_last_err();
int 	TE54X_err_flag();
int 	TE54X_clr_err_flag();

		//--- returned error codes
#define	TE54X_OK			0	// no errors
#define	TE54X_PARAM_ERR			-1	// erroneous function parameter
#define	TE54X_FLASH_INV_DEV_ERR		-100	// unsupported FLASH device
#define	TE54X_FLASH_ERASE_ERR		-101	// FLASH erase error
#define	TE54X_FLASH_WR_ERR		-102	// FLASH write error
#define	TE54X_FLASH_CANNOT_WR_ERR	-103	// FLASH contents does not allow to write this data
#define	TE54X_FLASH_INV_RD_ERR		-104	// invalid FLASH read data during AUTO-SELECT

//================================================================================



/*=============== TE54x DSP Utility FUnction Bodies ==============================
 TE54x DSP function groups include functions for DSP W/S registers init and FLASH 
 programming control. 

 However, in order to reduce the size of DSP code and to exclude mutiple 
 function body appearance, any of the utility function groups can be included/excluded 
 during the compilation process. Inclusion/exclusion of particular DSP utility function
 group is controled via global vars, which can be defined in user code to perferm a 
 particular action:

	__TE54X_EXCLUDE_INIT_DSP__ 
		- if defined in user code, then the E54x and DSP w/s INIT functions 
		  are excluded from the user source code. Default is INCLUDED.

	__TE54X_INCLUDE_AMD_FLASH_CONTROL__ 
		- if defined in user code, then the AMD FLASH PROGRAMMING CONTROL function 
		  are included into the user source code. Default is EXCLUDED.
*/


#ifndef __TE54X_EXCLUDE_INIT_DSP__
//--------------- DSP INIT function ------------------------------
// These functions perform automatic initialization of E54x DSP configiration registers
// for different E54X boards. DSP XF flag is set to '1' and enables non-PIOX peripherals
// and control registers.

#define	__C54X_SWWSR__	0x28
#define	__C54X_BSCR__	0x29
#define	__C54X_SWCR__	0x2B
#define	__C54X_CLKMD__	0x58
#define	__C54X_HPIC__	0x2C
#define	__C54X_IMR__	0x00
#define	__C54X_IFR__	0x01
#define	__C54X_HPIC__	0x2C



void TE54X_init(unsigned xdm_sel)
// Performs init of E54x control registers and DSP on-chip IRQ/WS control regs.
// Inits all DSP WS control regs depending upon the input XDM_SEL parameter:
//   xdm_sel==TE54X_XDM_SEL_DRAM:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with SRAM as external D-MEM. External FLASH/EPROM cannot be accessed 
//	  after execution of this function.
//   xdm_sel==TE54X_XDM_SEL_FLASH:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with FLASH/EPROM as external D-MEM. External SRAM can be also accessed 
//	  after execution of this function, but at the lower performance (2ws instead 
//	  of 1ws).
//   xdm_sel==TE54X_XDM_SEL_AUTO:
//	  XDMP_RG is left unchanged and XDM_SEL bit of XDMP_RG is used to define
//	  external D-MEM memory type and to configure DSP on-chip WS control regs 
//	  correspondingly.
{
	// initialize DSP on-chip WS control regs
	TE54X_init_dsp(xdm_sel);

	// set XDM page #0
	TE54X_set_xdm_page(0);

	// disable all external interrupts
	TE54X_set_mnmi_sel(TE54X_MIRQ_SEL_OFF);
	TE54X_set_mirq0_sel(TE54X_MIRQ_SEL_OFF);
	TE54X_set_mirq1_sel(TE54X_MIRQ_SEL_OFF);
	TE54X_set_mirq2_sel(TE54X_MIRQ_SEL_OFF);
	TE54X_set_mirq3_sel(TE54X_MIRQ_SEL_OFF);

	// disable WDT
	TE54X_set_wdt_enable(TE54X_WDT_ENABLE_OFF);

	// set active PX/SX reset
	TE54X_set_pxsx_reset_rg(TE54X_PX_RESET_ON | TE54X_SXA_RESET_ON |TE54X_SXB_RESET_ON);

	// configure T/XIO-0/1 as I/O@INPUT pins
	TE54X_set_xio_cnf_rg(TE54X_XIO0_CNF_IO | TE54X_XIO1_CNF_IO);
	TE54X_set_xio_dir_rg(TE54X_XIO0_DIR_INPUT | TE54X_XIO1_DIR_INPUT);

	// user can add application specific init of USART, USB, McBSP and other
	// DSP peripherals here ...
}



void TE54X_init_dsp(unsigned xdm_sel)
// Performs init of DSP on-chip IRQ control regs and all DSP WS control regs 
// depending upon the input XDM_SEL parameter:
//   xdm_sel==TE54X_XDM_SEL_DRAM:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with SRAM as external D-MEM. External FLASH/EPROM cannot be accessed 
//	  after execution of this function.
//   xdm_sel==TE54X_XDM_SEL_FLASH:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with FLASH/EPROM as external D-MEM. External SRAM can be also accessed 
//	  after execution of this function, but at the lower performance (2ws instead 
//	  of 1ws).
//   xdm_sel==TE54X_XDM_SEL_AUTO:
//	  XDMP_RG is left unchanged and XDM_SEL bit of XDMP_RG is used to define
//	  external D-MEM memory type and to configure DSP on-chip WS control regs 
//	  correspondingly.
{
  *(unsigned *)__C54X_IMR__  =0;		// clear IMR since it is not cleared on reset
  *(unsigned *)__C54X_IFR__  =0xffff;		// clear all interrupt requests
  *(unsigned *)__C54X_HPIC__ =0x2;		// INIT HPIC, setup shared mode for HPI area (C548/C549)

	// select non-PIOX I/O area	
  TE54X_SET_NOTPX_IO_AREA;

	// set XDMP_RG in case xdm_sel is either TE54X_XDM_SEL_DRAM or TE54X_XDM_SEL_FLASH
  if(   xdm_sel==TE54X_XDM_SEL_DRAM 
     || xdm_sel==TE54X_XDM_SEL_FLASH
    )
    TE54X_set_xdm_sel(xdm_sel);

  switch(TE54X_get_dev_id())
  { 
    case TE548_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE548_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE548_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE548_SWWSR_DATA_DRAM:TE548_SWWSR_DATA_FLASH
					 )
					)
				      );
	 *(unsigned *)__C54X_BSCR__  =TE548_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE548_CLKMD_DATA;		
	break;

    case TE549_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE549_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE549_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE549_SWWSR_DATA_DRAM:TE549_SWWSR_DATA_FLASH
					 )
					)
				      );		
	 *(unsigned *)__C54X_SWCR__  =TE549_SWCR_DATA;		
	 *(unsigned *)__C54X_BSCR__  =TE549_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE549_CLKMD_DATA;		
	 *(unsigned *)__C54X_HPIC__  =0x2;	// setup shared mode for HPI area
	break;

    case TE5402_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5402_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5402_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5402_SWWSR_DATA_DRAM:TE5402_SWWSR_DATA_FLASH
					 )
					)
				      );		
	 *(unsigned *)__C54X_SWCR__  =TE5402_SWCR_DATA;		
	 *(unsigned *)__C54X_BSCR__  =TE5402_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE5402_CLKMD_DATA;		
	break;

    case TE5409_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5409_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5409_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5409_SWWSR_DATA_DRAM:TE5409_SWWSR_DATA_FLASH
					 )
					)
				      );		
	 *(unsigned *)__C54X_SWCR__  =TE5409_SWCR_DATA;		
	 *(unsigned *)__C54X_BSCR__  =TE5409_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE5409_CLKMD_DATA;		
	break;

    case TE5410_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5410_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5410_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5410_SWWSR_DATA_DRAM:TE5410_SWWSR_DATA_FLASH
					 )
					)
				      );		
	 *(unsigned *)__C54X_SWCR__  =TE5410_SWCR_DATA;		
	 *(unsigned *)__C54X_BSCR__  =TE5410_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE5410_CLKMD_DATA;		
	break;

    case TE5416_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5416_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5416_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5416_SWWSR_DATA_DRAM:TE5416_SWWSR_DATA_FLASH
					 )
					)
				      );		
	 *(unsigned *)__C54X_SWCR__  =TE5416_SWCR_DATA;		
	 *(unsigned *)__C54X_BSCR__  =TE5416_BSCR_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE54X_CLKMD_PRESET_DATA;		
	 *(unsigned *)__C54X_CLKMD__ =TE5416_CLKMD_DATA;		
	break;

	// get here on incorrect DEV ID, exit
    default:
	break;
  }
}




void TE54X_reinit_dsp(unsigned xdm_sel)
// Performs run-time reinit of DSP SWWSR reg only depending upon the input 
// XDM_SEL parameter in case run-time switching between external D-RAM and FLASH is
// required:
//   xdm_sel==TE54X_XDM_SEL_DRAM:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with SRAM as external D-MEM. External FLASH/EPROM cannot be accessed 
//	  after execution of this function.
//   xdm_sel==TE54X_XDM_SEL_FLASH:
//	  XDMP_RG and DSP on-chip WS control regs are configured for operation 
//	  with FLASH/EPROM as external D-MEM. External SRAM can be also accessed 
//	  after execution of this function, but at the lower performance (2ws instead 
//	  of 1ws).
//   xdm_sel==TE54X_XDM_SEL_AUTO:
//	  XDMP_RG is left unchanged and XDM_SEL bit of XDMP_RG is used to define
//	  external D-MEM memory type and to configure DSP on-chip SWWSR reg
//	  correspondingly.
{
	// select non-PIOX I/O area	
  TE54X_SET_NOTPX_IO_AREA;

	// set XDMP_RG in case xdm_sel is either TE54X_XDM_SEL_DRAM or TE54X_XDM_SEL_FLASH
  if(   xdm_sel==TE54X_XDM_SEL_DRAM 
     || xdm_sel==TE54X_XDM_SEL_FLASH
    )
    TE54X_set_xdm_sel(xdm_sel);

  switch(TE54X_get_dev_id())
  { 
    case TE548_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE548_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE548_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE548_SWWSR_DATA_DRAM:TE548_SWWSR_DATA_FLASH
					 )
					)
				      );
	break;

    case TE549_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE549_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE549_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE549_SWWSR_DATA_DRAM:TE549_SWWSR_DATA_FLASH
					 )
					)
				      );		
	break;

    case TE5402_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5402_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5402_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5402_SWWSR_DATA_DRAM:TE5402_SWWSR_DATA_FLASH
					 )
					)
				      );		
	break;

    case TE5409_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5409_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5409_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5409_SWWSR_DATA_DRAM:TE5409_SWWSR_DATA_FLASH
					 )
					)
				      );		
	break;

    case TE5410_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5410_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5410_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5410_SWWSR_DATA_DRAM:TE5410_SWWSR_DATA_FLASH
					 )
					)
				      );		
	break;

    case TE5416_DEV_ID:
	 *(unsigned *)__C54X_SWWSR__ =((xdm_sel==TE54X_XDM_SEL_DRAM)? TE5416_SWWSR_DATA_DRAM:
					((xdm_sel==TE54X_XDM_SEL_FLASH)? TE5416_SWWSR_DATA_FLASH:
					 ((TE54X_XDM_SEL_IS_DRAM)? TE5416_SWWSR_DATA_DRAM:TE5416_SWWSR_DATA_FLASH
					 )
					)
				      );		
	break;

	// get here on incorrect DEV ID, exit
    default:
	break;
  }
}

//---------------------------------------------------------------------
#endif	// DSP INIT



#ifdef __TE54X_INCLUDE_AMD_FLASH_CONTROL__
/*-------------- general extended API functions -------------------------
    GENERAL EXTENDED API FUNCTIONS shall be included in case of any extended
    API functions have been included.

    General extended TE54X API functions are used to handle the run-time
    errors for FLASH programming control and other extended API functions:
	TE54X_last_err
	TE54X_err_flag
	TE54X_clr_err_flag

    Each function returns error code (<0). In case the function must also
    return data (as TE54X_flash_read()), then data can be recognized in case
    returned code is >=0, and error is recognized in case returned code is
    <0.
*/

static	int _te54x_last_err =0,
	    _te54x_err_flag =0;


int TE54X_last_err()
	// TE54X_LAST_ERR function returns errpr code of last executed
	// API function.
 { return(_te54x_last_err);
 }


int TE54X_err_flag()
	// TE54X_ERR_FLAG function returns status of ERR_FLAG for API
	// functions. ERR_FLAG is set by each API function in case
	// error has been detected. ERR_FLAG can be reset by
	// TE54X_CLR_ERR_FLAG API function
 { return(_te54x_err_flag);
 }

int TE54X_clr_err_flag()
 {
   _te54x_err_flag=TE54X_OFF;
   return(TE54X_OK);
 }


	// local/non-API function,
	// used for exit fm API fucntions to set error code
static int _te54x_set_err(int e)	
 {
   if(e<0)
     { _te54x_err_flag=1;
       _te54x_last_err=e;
     }
     else
       _te54x_last_err=TE54X_OK;
   return(e);
 }

//---------------------------------------------------------------------
#endif	// general XAPI




#ifdef __TE54X_INCLUDE_AMD_FLASH_CONTROL__
/*--------------- AMD FLASH PROGRAMMING CONTROL functions --------------
   IMPORTANT NOTES !!!
   ---------------------
    1. AMD FLASH control utilities for TE54X have been designed for AMD Am29F010B/Am29F040B 
       5v-only FLASH memory chips only (PLCC-32 IC package) and will not run perfectly 
       with FLASH memory chips rom other manufacturers.

    2. FLASH control utilities (except for the TE54X_flash_read() and TE54X_flash_set_page() 
       functions) require that FLASH WRITE feature is enabled via on-board SW3 switch,
       oherwise the TE54X_FLASH_INVALID_ERR error will be returned and no operation 
       will be performed. This is due FLASH control utilies are auto-configured via the 
       FLASH MANUF/DEVICE info, which is read from the FLASH chip using AUTO-SELECT command.

    3. User FLASH API functions:
	    	TE54X_flash_info
		TE54X_flash_set_page
		TE54X_flash_erase_chip
        	TE54X_flash_erase_sector
		TE54X_flash_write
		TE54X_flash_read
		TE54X_flash_sector_prot

    4. At runtime, in case FLASH WRITE feature is enabled, FLASH utilities automatically 
       detect presence of the supported FLASH memory chips, and adjusts internal environment.

    5. Upon return, all FLASH utilities select FLASH memory as external data memory
       via XDMP_RG register and set NON-PIOX I/O area, which corresponds to DSP XF=1.
*/

#include "stddef.h"	// TI C Compiler standard header


#define _FLASH_AM29F010_LEN		0x20000L
#define _FLASH_AM29F040_LEN		0x80000L
#define _FLASH_AM29F010_SECTORS		8
#define _FLASH_AM29F040_SECTORS		8
#define _FLASH_AM29F_SECTOR_PROT_ID	1
#define _FLASH_AM29F_SECTOR_UNPROT_ID	0


static int	_te54x_flash_open_flag	= 0;
static int	_te54x_flash_manuf_id	= 0,
		_te54x_flash_dev_id	= 0;
static long	_te54x_flash_len	= 0,
		_te54x_flash_sectors	= 0,
		_te54x_flash_sector_len = 0;


static void _te54x_flash_set_page_ptr(unsigned long a, unsigned *fpage, unsigned **fptr)
	// local/non-API function, which is used to calculate FLASH page and PTR.
	// XDMP_RG is set to XDM_SEL_FLASH and calculated FLASH PAGE.
{ 
	unsigned _fpage;

	// calculate and set the XDMP rg to address the FLASH page
	_fpage = (unsigned) ((unsigned long) (a/TE54X_FLASH_PAGE_LEN));

	// select conrol regs area
	TE54X_SET_NOTPX_IO_AREA;

	// load XDMP_RG register
    TE54X_set_xdmp_rg(TE54X_XDM_SEL_FLASH | _fpage);

	// return FPAGE if requested
	if(fpage!=NULL)  *fpage = _fpage;

	// return FPTR if requested
	if(fptr!=NULL)
		*fptr = (unsigned *)(TE54X_FLASH_PAGE_SADDR + (unsigned)((unsigned long)(a%TE54X_FLASH_PAGE_LEN)));
}


static int _te54x_flash_open()
	// local/non-API function, which is used to check whether
	// FLASH API has been open in order to identify FLASH chip info
	// and configure FLASH API environment
{
	unsigned *fptr,rd;
 	int r;

  	// Check for FLASH API has been open already.
	// If not open, then open it.
    if (_te54x_flash_open_flag==TE54X_ON) return(TE54X_OK);	// exit if already opened

	// not open yet, now try to open FLASH utilities
    _te54x_flash_open_flag=TE54X_OFF;

	// set FLASH page, which corresponds to FLASH address 0.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);

    // execute AUTO-SELECT command
    fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0x90;
    _te54x_flash_manuf_id = fptr[0] & TE54X_FLASH_DATA_BITMASK;

    r=TE54X_FLASH_INV_DEV_ERR;	// assume error
    if(_te54x_flash_manuf_id != TE54X_FLASH_AMD_MANUF_ID) goto exit;

    // now read and verify the device ID
	//-- list of supported FLASHs can be extended !!!
    _te54x_flash_dev_id = fptr[1] & TE54X_FLASH_DATA_BITMASK;

	r=TE54X_FLASH_INV_DEV_ERR;	// assume error
    if(!(    (_te54x_flash_manuf_id == TE54X_FLASH_AMD_MANUF_ID)
		  && (    	_te54x_flash_dev_id == TE54X_FLASH_AM29F010_DEV_ID
		  		|| 	_te54x_flash_dev_id == TE54X_FLASH_AM29F040_DEV_ID
             )
	    )
	  )
		goto exit;

	// now fill-in FLASH len , number of sectors, & sector len
    if(_te54x_flash_manuf_id == TE54X_FLASH_AMD_MANUF_ID)
   	{
	 	switch(_te54x_flash_dev_id)
	    {
        	case TE54X_FLASH_AM29F010_DEV_ID:
				_te54x_flash_len	= _FLASH_AM29F010_LEN;
				_te54x_flash_sectors	= _FLASH_AM29F010_SECTORS;
				_te54x_flash_sector_len = _FLASH_AM29F010_LEN/_FLASH_AM29F010_SECTORS;
				break;

            case TE54X_FLASH_AM29F040_DEV_ID:
				_te54x_flash_len	= _FLASH_AM29F040_LEN;
				_te54x_flash_sectors	= _FLASH_AM29F040_SECTORS;
				_te54x_flash_sector_len = _FLASH_AM29F040_LEN/_FLASH_AM29F010_SECTORS;
				break;
		}
	}

	// That's it, now set FLASH utilities OPEN flag
	// in order to exclude this procedure for all further
	// API function calls
    _te54x_flash_open_flag=TE54X_ON;
	r=TE54X_OK;
exit:
	// execute READ/RESET command
	// in order to enable further reads
    fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0xf0;
    rd = *fptr;		// perform false read
    rd &= 0;		// perform false operation in order to exclude warning message from the compiler
   	return(r);		// return error code
}


int TE54X_flash_set_page(unsigned long a, unsigned *fpage, unsigned **fptr)
	// set the XDMP rg in accordance with FLASH address A,
	// calculates and returns PTR to access the addressed FLASH location
	// returns: =0  - no error
	//          <0  - error
	// Note: TE54X_flash_set_page() function does not generally require
	//       FLASH WRITE enabled via switch SW3 in order to have FLASH
	//	 utilities running perfectly. In case FLASH utilities are open,
	//	 then A address is verified againg particular FLASH length,
	//	 otherwise  maximum FLASH/EPROM length is used.
{
	// execute FLASH OPEN procedure in order to activate
	// FLASH utilities, however ignore returned error code
	_te54x_flash_open();

	// validate function params
	if (     (_te54x_flash_open_flag==TE54X_ON  && a>=_te54x_flash_len)
	      || (_te54x_flash_open_flag==TE54X_OFF && a>=TE54X_FLASH_LEN)
	   )
		return(_te54x_set_err(TE54X_PARAM_ERR));

	// calculate/return FLASH page/ptr, load XDMP reg
	_te54x_flash_set_page_ptr(a,fpage,fptr);

	// exit with no error
	return(_te54x_set_err(TE54X_OK));
}



int TE54X_flash_info(unsigned *manuf_id, unsigned *dev_id,
		    unsigned long *len,unsigned long *sectors, unsigned long *sector_len)
	// Returns FLASH chip info (manuf code, device code, etc.) via
	// suplied params pointers.
	// Returns: error code.
{
	int r;

  	// Check for FLASH API has been open already.
	 if ((r=_te54x_flash_open())!=TE54X_OK) return(_te54x_set_err(r));

	if(manuf_id!=NULL) 		*manuf_id   =_te54x_flash_manuf_id;
	if(dev_id!=NULL) 		*dev_id     =_te54x_flash_dev_id;
	if(len!=NULL) 			*len	    =_te54x_flash_len;
	if(sectors!=NULL)		*sectors    =_te54x_flash_sectors;
	if(sector_len!=NULL)	*sector_len =_te54x_flash_sector_len;

	// exit with no error
  	return(_te54x_set_err(TE54X_OK));
}


int TE54X_flash_sector_wrprot(unsigned long a)
	// Returns FLASH sector write-protection status for the sector,
	// which corresponds to address A.
	// Returns: <0 error code.
	//          0  - sector is not protected
	//          1  - sector is protected
{ 
	int r;
  	unsigned *fptr,rd,rd1;

  	// Check for FLASH API has been open already.
	if ((r=_te54x_flash_open())!=TE54X_OK) return(_te54x_set_err(r));

	// validate params
	if (a>=_te54x_flash_len) return(_te54x_set_err(TE54X_PARAM_ERR));

	// set FLASH page, which corresponds to FLASH address 0.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);

	// execute AUTO-SELECT COMMAND for sector protection status
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x90;

	// read & verify sector wr-protection status (==0 or ==1)
	// set FLASH page, which corresponds to the sector for FLASH address #A.
	r=TE54X_FLASH_INV_RD_ERR;	// assume error initially
    _te54x_flash_set_page_ptr(((a/_te54x_flash_sector_len)*_te54x_flash_sector_len),NULL,&fptr);
	rd= fptr[2] & TE54X_FLASH_DATA_BITMASK;
    if(rd!=0 && rd!=1) goto exit;

	r=TE54X_OK;	// exit OK
exit:
	// set FLASH page, which corresponds to FLASH address 0.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);

    // execute READ/RESET command in order to enable further reads
    fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0xf0;
    rd1 = *fptr;	// perform false read
    rd1 &= 0;		// perform false operation in order to exclude warning message from the compiler

    return(_te54x_set_err(r<0? r:rd));
}



int TE54X_flash_erase_sector(unsigned long a,unsigned vf)
	// Erase sector, which corresponds to FLASH address A (0...
	// Verify erased sector data if VF==1.
	// returns:  error code
{ 
	unsigned *fptr,rd;
  	int r;
  	unsigned long i,faddr;

  	// Check for FLASH API has been open already.
	if ((r=_te54x_flash_open())!=TE54X_OK) return(_te54x_set_err(r));

	// validate params
	if (   a>=_te54x_flash_len
	    || vf>1
	   )
	   return(_te54x_set_err(TE54X_PARAM_ERR));

	// set FLASH page, which corresponds to FLASH address 0.
	// save fptr, which corresponds to FLASH address 0.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);

	// execute SECTOR ERASE command
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x80;
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;

	// now set FLASH page, which corresponds to FLASH address A
	// and write READ FLASH WR-PROT status for sector
    _te54x_flash_set_page_ptr(a,NULL,&fptr);
	*fptr = 0x30;

    while(1)
	{ 
		rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;

		// verify read data
	  	if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;
            if(rd==0xff) break;	// exit OK
			return(_te54x_set_err(TE54X_FLASH_ERASE_ERR));	// error and exit
	    }
	}

		// perform erased sector data verification if requested
    if(vf!=0)
	{
		// Set FLASH page, which corresponds to start of sector for FLASH address A.
		// Sector len may exceed XDM page len, so FLASH page boundary must be tracked.
		// Set flash addr to start of sector.
	   	faddr = ((a/_te54x_flash_sector_len)*_te54x_flash_sector_len);
	   	_te54x_flash_set_page_ptr(faddr,NULL,&fptr);

	   	for(i=0L;i<_te54x_flash_sector_len;i++)
		{
			if(((*fptr) &TE54X_FLASH_DATA_BITMASK)!=0xff)
		    	return(_te54x_set_err(TE54X_FLASH_ERASE_ERR));

		  	// increment FADDR and check for FLASH page boundary cross
		  	// update FLASH page/ptr on boundary cross
		  	if(((++faddr)%TE54X_FLASH_PAGE_LEN)==0)
				_te54x_flash_set_page_ptr(faddr,NULL,&fptr);
		  	else
				fptr++;	// otherwise increment fptr
		}
	 }

	// exit OK
	return(_te54x_set_err(TE54X_OK));
}



int TE54X_flash_erase_chip(unsigned vf)
	// Erase all sectors of FLASH. Verify erased data if VF==1.
	// returns:  error code
{ 
	unsigned *fptr,rd;
  	int r;
  	unsigned long i,faddr;

  	// Check for FLASH API has been open already.
	if ((r=_te54x_flash_open())!=TE54X_OK) return(_te54x_set_err(r));

	// validate params
	if (vf>1) return(_te54x_set_err(TE54X_PARAM_ERR));

	// set FLASH page, which corresponds to FLASH address 0.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);

	// execute ERASE CHIP command
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x80;
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
	fptr[0x5555]=0x10;

    while(1)
	{ 
		rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;

		// verify read data
	  	if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;
            if(rd==0xff) break;	// exit OK
			return(_te54x_set_err(TE54X_FLASH_ERASE_ERR));	// error and exit
	    }
	}
	
	// perform erased chip data verification if requested
    if(vf!=0)
	{
		// Set FLASH page, which corresponds to FLASH address 0.
		// Check for FLASH page boundary crossing.
	   	faddr = 0L;
	   	_te54x_flash_set_page_ptr(faddr,NULL,&fptr);

	   	for(i=0L;i<_te54x_flash_len;i++)
		{
			if(((*fptr) &TE54X_FLASH_DATA_BITMASK)!=0xff)
		    	return(_te54x_set_err(TE54X_FLASH_ERASE_ERR));

		  	// increment FADDR and check for FLASH page boundary cross
		  	// update FLASH page/ptr on boundary cross
		  	if(((++faddr)%TE54X_FLASH_PAGE_LEN)==0)
				_te54x_flash_set_page_ptr(faddr,NULL,&fptr);
		  	else
				fptr++;	// otherwise increment fptr
		}
	}

	// exit OK
	return(_te54x_set_err(TE54X_OK));
}


int TE54X_flash_write(unsigned long a, unsigned d)
	// writes D (bits 0..7 are valid) to FLASH addr #A
	// returns:  error code
{ 
	unsigned *fptr,rd;
  	int r;

  	// Check for FLASH API has been open already.
	if ((r=_te54x_flash_open())!=TE54X_OK) return(_te54x_set_err(r));

	// validate params
	if (   a>=_te54x_flash_len
	    || d>TE54X_FLASH_DATA_BITMASK
	   ) 
	   return(_te54x_set_err(TE54X_PARAM_ERR));

	// verify that FLASH contents at address A
	// is OK to program data D, i.e. '1' bits in D
    _te54x_flash_set_page_ptr(a,NULL,&fptr);
	if (((*fptr) & d)!=d)
		return(_te54x_set_err(TE54X_FLASH_CANNOT_WR_ERR));

	// execute PROGRAM BYTE command
	// set FLASH page, which corresponds to FLASH address 0, and execute WRITE command sequence.
    _te54x_flash_set_page_ptr(0L,NULL,&fptr);
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0xa0;
	
	// set FLASH page to address A.
    _te54x_flash_set_page_ptr(a,NULL,&fptr);
    *fptr =d ;

	// verify for program ends and errors
    while(1)
	{ 
		rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;

		// verify read data
	  	if(rd==d) return(_te54x_set_err(TE54X_OK));		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = (*fptr) & TE54X_FLASH_DATA_BITMASK;
            if(rd==d) return(_te54x_set_err(TE54X_OK));	// exit OK
			return(_te54x_set_err(TE54X_FLASH_WR_ERR));	// error and exit
	    }
	}
}


int TE54X_flash_read(unsigned long a)
	// reads/returns data from FLASH addr #A
	// returns: <0 - error occured
	//	    >=0 - FLASH read data
	//
	// Note: TE54X_FLASH_READ function does not generally require
	//       FLASH WRITE enabled via switch SW3 in order to
	//       have FLASH utilities running perfectly.
	//       In case FLASH utilities are open, then A address is
	//       verified againg particular FLASH length, otherwise
	//       maximum FLASH/EPROM length is used.

{
	unsigned *fptr;

	// execute FLASH OPEN procedure in order to activate
	// FLASH utilities, however ignore returned error code
	_te54x_flash_open();

	// validate function params
	if (    (_te54x_flash_open_flag==TE54X_ON  && a>=_te54x_flash_len)
	      ||(_te54x_flash_open_flag==TE54X_OFF && a>=TE54X_FLASH_LEN)
	   )
	   return(_te54x_set_err(TE54X_PARAM_ERR));

	// assume no error
	_te54x_set_err(TE54X_OK);

	// set FLASH page to address A and read data
    _te54x_flash_set_page_ptr(a,NULL,&fptr);
	return((*fptr) & TE54X_FLASH_DATA_BITMASK);
}


//---------------------------------------------------------------------
#endif	// FLASH

#endif	// __TE54X_DEF__
